#####
# Sample identification and treatment randomization
# 2021 ETH+Stadt Zurich naturalization-outreach evaluation
#####

## Set working directories
data_raw_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data_raw/")
data_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data/")
graphs_path <- ("/Users/kahrens/MyProjects/policy_tree/Outputs/randomization/")


## Load packages
library(data.table)
library(lubridate)
library(ggplot2)
library(stringi)
library(utils)
library(randomizr)
library(lfe)
library(openxlsx)


## Set seed
seed <- 76543 


## Load data
zh <- read.xlsx(paste0(data_raw_path,"Datenbekanntgabe Einbügerungsversand 2021.xlsx"))

#####
# Preliminaries
#####
save_names <- colnames(zh)

# set better names
setnames(
  zh, 
  old = colnames(zh),
  new = c(
    "PID",
    "greeting",
    "last_name",
    "given_name",
    "first_name",
    "name_supplement",
    "street",
    "house_number",
    "PLZ",
    "city",
    "gender",
    "year-of-birth",
    "dob",
    "arrive_CH",
    "arrive_ZH",
    "nat"
  )
)



# convert to data.table
zh <- as.data.table(zh)

# Crosswalk country-codes and country names, because the other file (submitted applications) is in country names
codes_names <- fread(paste0(data_raw_path,"crosswalk-country-name-code-for-randomisation.csv"), header=TRUE)
codes_names <- codes_names[,.(nat, name)]
setnames(codes_names, old="name", new="country_name")

zh <- merge(zh, codes_names, by="nat", all.x=TRUE)   #Merge
setnames(zh, old="nat", new="iso_code") 
setnames(zh, old="country_name", new="nat")  # Rename variable such that we can merge with other file later


Encoding(zh$nat) <- "UTF-8"
Encoding(zh$last_name) <- "UTF-8"
Encoding(zh$first_name) <- "UTF-8"

# change date variables into easily used format
zh[ , dob := ymd(dob)]
zh[ , arrive_CH := ymd(arrive_CH)]
zh[ , arrive_ZH := ymd(arrive_ZH)]



#####
# Drop 3 people who don’t want their data to be given out by the city :
# 101745298  
# 101787882 
# 101788548 
zh <- zh[PID!=101745298]
zh <- zh[PID!=101787882]
zh <- zh[PID!=101788548]

# Drop one person who was included twice, PID=101525028
zh <- unique(zh)

#####
# Drop people who received the 2019 letter.
# Must meet all the below criteria:
# 1. Date of birth: 01.01.2002-30.06.2003   # 18 years old or older (from 2019 letter)
# --> TO DROP: dob <= 31.12.2001         
# 2. Date of entry Switzerland: 01.01.2010-30.06.2011   # at least 10 years in Switzerland (from 2019 letter)
# --> TO DROP: arrive_CH <= 31.12.2009   
# 3. Arrival date Zurich: 01.01.2018-30.06.2019   # at least 2 years in Zurich (from 2019 letter)
# --> TO DROP: arrive_ZH <= 31.12.2017   


# Add new column "letter_1", the data.table way: :=
# Is TRUE (=1) if these conditions are met (and therefore will be dropped). I.e. is true if HAS received 2019 letter already
zh[ , letter_1 :=
      dob <= ymd("2001-12-31") &
      arrive_CH <= ymd("2009-12-31") &
      arrive_ZH <= ymd("2017-12-31")
    ]
#---> There are 339 observations out of 6207 (5.4%) that have to be dropped

setkey(zh, letter_1)
dropped_2019 <- zh[.(T)]  # Check these
fwrite(dropped_2019, paste(data_path, "dropped_2019.csv"))
zh <- zh[.(F)] # Here we are keeping only FALSE observations -- meaning those who did not yet receive the 2019 letter
dim(zh) # =5868 (In fact 6207-339=5868)

zh[, letter_1 := NULL]


#####
# Drop people that have submitted an application in 2020-2021 (until date 21.09.2021)
#####

# The files were sent from the City of Zurich in two different moments 
# From 01.01.2020-05.08.2021
gesuchs1 <- read.xlsx(paste0(data_raw_path,"Gesuchseingäng SKZ 01.01.2020-05.08.2021.xlsx"),"Personendaten", detectDates = TRUE)

# From 06.08.2021-21.09.202
gesuchs2 <- read.xlsx(paste0(data_raw_path,"Gesuchseingänge SKZ 06.08.2021-21.09.2021.xlsx"),"Personendaten", detectDates = TRUE)

# Append and clean
gesuchs <- rbind(gesuchs1, gesuchs2)
rm(gesuchs1, gesuchs2)

# Convert to data.table
gesuchs <- as.data.table(gesuchs)


# Check that they are not applicants or children
setkey(gesuchs, Rolle)
gesuchs <- gesuchs[ .(c("Bewerber / Bewerberin", "Kind"))]
# ---> 14 people were dropped because they were not applicants nor children

# Easier variable names
setnames(
  gesuchs, 
  old = c(
    "Nachname",
    "Rufname",
    "Strasse",
    "Geburtsdatum",
    "Staatsangehörigkeit",
    "Geschlecht",
    "Wohnsitzfrist:.CH.seit",
    "Wohnsitzfrist:.ZH.seit"
  ),
  new = c(
    "last_name",
    "first_name",
    "street_house_address",
    "dob",
    "nat",
    "gender",
    "arrive_CH",
    "arrive_ZH"
  )
)


# Fix the country names that are coded differently
corrections <- c(
  "Vereinigte Staaten" = "USA",
  "Dominikanische Republik" = "Dominik. Rep.",
  "Kongo (Kinshasa)" = "Kongo, Dem. Rep",
  "Mazedonien" = "Nordmazedonien",
  "Vereinigtes Königreich" = "Grossbritannien",
  "Moldova" = "Moldau",
  "Serbien und Montenegro" = "Serbien",
  "Tschechische Republik" = "Tschechische R.",
  "Bosnien und Herzegowina" = "Bosnien-Herz.",
  "China" = "China, Volksr.",
  "Staatsangehörigkeit ungeklärt" = "Unbekannt",
  "Tschechien" = "Tschechische R.",
  "Eritre" = "Eritrea",
  "Deu" = "Deutschland",
  "Deutschlan" = "Deutschland"
)

gesuchs[ nat %in% names(corrections), nat := corrections[nat]]

# Reformat date of birth
gesuchs[ , dob := dmy(dob)] 

# 1. As with other file, identify people that received letter 1 (2019) - to be DROPPED
# Add new column "letter_1"
# Is TRUE (=1) if these conditions are met (and therefore will be dropped). I.e. is true if HAS received 2019 letter already
gesuchs[ , letter_1 :=
      dob <= ymd("2001-12-31") &
      arrive_CH <= ymd("2009-12-31") &
      arrive_ZH <= ymd("2017-12-31")
]
gesuchs[ , table(letter_1, useNA = "always")]

# ---> There are 2607 observations out of 5918 that have to be dropped. There are 309 missings

setkey(gesuchs, letter_1)
gesuchs <- gesuchs[.(c(FALSE, NA))] # Here we are keeping only FALSE observations -- meaning those who did not yet receive the 2019 letter
dim(gesuchs) # =3311 (In fact 5918-2607=3311)
gesuchs[ , letter_1 := NULL]


# 2. Check: identify people that qualify for this experiment (2021) - to be KEPT
gesuchs[ , letter_2 := arrive_CH <= "2011-06-30" & arrive_ZH <= "2019-06-30" & dob <= "2003-06-30"]
gesuchs[ , table(letter_2, useNA = "always")]


setkey(gesuchs, letter_2)
gesuchs <- gesuchs[.(c(TRUE, NA))]
gesuchs[ , letter_2 := NULL]
# ---> There are 2156 observations that have to be dropped because they applied but they don't qualify for the citizenship 

check <- gesuchs[ , .(arrive_CH,arrive_ZH,dob)]
rm(check)
dim(gesuchs) # =1155 (In fact, 3311-2156=1155)


# If you have a Bürgerort (CH), you are already Swiss. Drop these people
setkey(gesuchs, `Bürgerort.(CH)`)
gesuchs <- gesuchs[.(NA)] # Keep only missing values, which correspond to those WITHOUT a passport
# ---> 254 dropped because they already had a passport
dim(gesuchs) # =901


## Matching, based on various characteristics
# do a version of names stripping out case and accents
gesuchs[ , first_name_S := stri_trans_general(first_name, "latin-ascii")]
gesuchs[ , last_name_S := stri_trans_general(last_name, "latin-ascii")]
zh[ , first_name_S := stri_trans_general(first_name, "latin-ascii")]
zh[ , last_name_S := stri_trans_general(last_name, "latin-ascii")]

# Look for matches based on simplified names
gesuchs[ , name_match_S := paste(last_name_S, first_name_S, sep = ", ") %in% zh[ , paste(last_name_S, first_name_S, sep = ", ")]]


test <- adist(x = gesuchs[ ,paste0(last_name_S, ", ", first_name_S, ", ", dob)], y = zh[,paste0(last_name_S, ", ", first_name_S, ", ", dob)])

test

dtf <- as.data.table(test)
best_matches <- apply(dtf, 1, function(x) which(x == min(x), useNames = F))
match_quality <- apply(dtf, 1, function(x) min(x))
reps <- sapply(best_matches, length)
gesuchs_cols <- unlist(mapply(rep, x = 1:gesuchs[,.N], times = reps, SIMPLIFY = T))
match_quals <- unlist(mapply(rep, x = match_quality, times = reps, SIMPLIFY = T))

multi_match <- 
  cbind(
    gesuchs[gesuchs_cols, .(
      "f_name_gs" = first_name_S,
      "l_name_gs" = last_name_S,
      "dob_gs" = dob,
      "name_match_S" = name_match_S,
      "nat_gs" = nat,
      "arrive_CH_gs" = arrive_CH,
      "arrive_ZH_gs" = arrive_ZH,
      "match_quals" = match_quals,
      "add_gs" = street_house_address
      )],
    zh[unlist(best_matches), .(
      "f_name_let" = first_name_S,
      "l_name_let" = last_name_S,
      "dob_let" = dob,
      "nat_let" = nat,
      "arrive_CH_let" = arrive_CH,
      "arrive_ZH_let" = arrive_ZH,
      "add_let" = paste0(street, ' ', house_number)
    )]
    )

# Create some variables to test matches
multi_match[ , dob_dist := abs(dob_gs - dob_let)]
multi_match[ , CH_dist := abs(arrive_CH_gs - arrive_CH_let)]
multi_match[ , ZH_dist := abs(arrive_ZH_gs - arrive_ZH_let)]
multi_match[ , nat_match := nat_gs == nat_let]
multi_match[ , N_matches := .N, by = .(f_name_gs, l_name_gs)]

# If a match is found, create an identifier
multi_match[ , match := FALSE]
multi_match[name_match_S == F & (dob_dist <= 5 & CH_dist <= 5 & ZH_dist <= 5), match := T]
multi_match[name_match_S == T  & !(dob_dist >= 100 & CH_dist  >= 100 & ZH_dist >= 100), match := T]

# Record matches in the letter and application
zh[ , matched := paste(first_name_S, last_name_S) %in% multi_match[match == T, paste(f_name_let, l_name_let)]]
gesuchs[ , match_2 := paste(first_name_S, last_name_S) %in% multi_match[match==T, paste(f_name_gs, l_name_gs)]]

# Check for matches using street addresses
zh[ , street_house_address := paste0(street, " ", house_number)]
gesuchs[match_2 == F, address_match := street_house_address %in% zh[ , street_house_address]]
setkey(gesuchs, street_house_address); setkey(zh, street_house_address)
house_matches <- zh[gesuchs[match_2 == F & address_match == T]]
house_matches[ , dob_dist := abs(`i.dob` - dob)]

# Add these matches to the application and letter data
gesuchs[paste0(first_name_S, last_name_S) %in% house_matches[dob_dist == 0, paste0(`i.first_name_S`, `i.last_name_S`)], match_2 := TRUE ]
zh[paste0(first_name_S, last_name_S) %in% house_matches[dob_dist == 0, paste0(`first_name_S`, `last_name_S`)], matched := TRUE ]

# Drop people who matched
zh <- zh[matched == F]
zh[ , matched := NULL]

#####
# Ensure all remaining people meet the 2021 criteria
# 1. dob <= 2003-06-30
# 2. arrive_CH <= 2011-06-30
# 3. arrive_ZH <= 2019-06-30
#####

zh[ , letter_2 := 
      dob <= ymd("2003-06-30") &
      arrive_CH <= ymd("2011-06-30") &
      arrive_ZH <= ymd("2019-06-30")
    ]

zh[ , table(letter_2)/.N] # all are eligible!
zh[ , letter_2 := NULL]



#####
# Create identifiers for the street-house number clusters
#####

zh[ , idx := .GRP, keyby = .(street, house_number)]
zh[ , .N, by = idx][ , table(N)]

clust_dist <- zh[, .N, by = idx][ , table(factor(N, levels = 1:11))]

ggplot(data = zh[ , .N, by = idx], aes(x = N)) +
  geom_bar(fill = "#ABC3C9", color = "black") +
  annotate("text", x = 1:11, y = as.numeric(clust_dist + 250), label = as.character(clust_dist)) +
  scale_x_continuous(breaks = c(1:12), expand = c(0,0.25)) +
  scale_y_continuous(expand = c(0,0), limits = c(0, 5900)) +
  xlab("Cluster size") +
  ylab("Count") +
  ggtitle("Distribution of Letter-Eligible Immigrants per Unique House Address") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank())

#####
# Record the primary origin in each cluster
#####

Modes <- function(x) {
  ux <- unique(x)
  tab <- tabulate(match(x, ux))
  ux[tab == max(tab)]
  
  if(length(ux) == 1){
    return(ux)
  }
  
  else{
    return(sample(ux, size = 1)) # when multiple nat, randomly select
  }
}

# how many unique nationalities per cluster? 
zh[ , uniqueN(nat), by = idx][ , table(V1)]




set.seed(seed)

# Create cluster variable
zh[ , cluster_nat := Modes(iso_code), by = idx]     


#####
# Randomize at the cluster level, blocking on nationality groups
#####


# Step 1. Create the blocks
zh[ , unique(nat)]



DE <- "DE" 
dev_dem <- c("AT", "GB", "FR", "FI", "LI", "SE", "NL", "IE", "BE", "DK", "NO", "LU", "IS", "US", "CA", "AU", "NZ")
EU_south <- c("IT", "ES", "PT", "GR", "CY")
EU_east <- c("AL", "SI", "MK", "HR", "XKO", "RS", "BA", "ME", "PL", "RO", "SK", "RU", "BG", "BY", "HU", "EE", "UA", "LT", "LV", "MD", "CZ")
rest_of_world <- c("IQ", "TR", "IR", "MA", "LB", "TN", "SD", "IL", "EG", "JO", "AF", "ZZX", "NA", "DZ", "SY", "SA", "LY", "AZ", "YE", "PS", "GE", "BD", "CN", "IN", "ID", "JP", "KH", "KZ", "KG", "KR", "MY", "MN", "NP", "PK", "PH", "SG", "LK", "TW", "TH", "UZ", "VN", "AO", "ET", "CI", "ER", "GM", "GH", "CM", "KE", "CD", "CG", "ML", "MU", "NG", "SN", "ZW", "SO", "ZA", "TZ", "TG", "CO", "BR", "JM", "AR", "DO", "BO", "CL", "PY", "CU", "PA", "MX", "VE", "GT", "EC", "HT", "CR", "UY", "PE", "NI", "MG", "HN", "MM")



zh[ , table(iso_code %in% c(DE, dev_dem, EU_south, EU_east, rest_of_world))] # check if all belong to a block. Expecting all observations to be TRUE
zh[ , table(sum(iso_code %in% c(DE, dev_dem, EU_south, EU_east, rest_of_world)))] # check if all belong to EXACTLY one block. Expecting all observations to be =1

# Assign to blocks
zh[ cluster_nat %in% DE, block := paste0("Germany, N=", prettyNum(.N, big.mark = ","))]
zh[ cluster_nat %in% dev_dem, block := paste0("Developed Democracies, N=", prettyNum(.N, big.mark = ","))]
zh[ cluster_nat %in% EU_south, block := paste0("Southern Europe, N=", prettyNum(.N, big.mark = ","))]
zh[ cluster_nat %in% EU_east, block := paste0("Central and Eastern Europe, N=", prettyNum(.N, big.mark = ","))]
zh[ cluster_nat %in% rest_of_world, block := paste0("Rest of World, N=", prettyNum(.N, big.mark = ","))]





zh[ , .N, by = block] 
zh[ , uniqueN(idx), by = block] # Table with: 1. number of people in each block, 2. number of clusters within each block


# Step 2. Randomize within blocks at the cluster level
# Here the randomization happens
# N = number of entities to be randomly assigned
# groupN = number of groups the entities are assigned to
cluster_permutation <- function(N, groupN){
  set.seed(seed)
  A <- rep(c(1:groupN), each = (N-(N %% groupN))/groupN) # closest number below N that is evenly divisible by groupN
  B <- sample(c(1:groupN), (N %% groupN)) # for the remainder, sample randomly from the groups
  randomized <- sample(c(A, B)) # sample randomly from this vector length N
  return(randomized)
}

# N corresponds to the number of clusters, since we randomize at the **cluster-level**
# groupN corresponds to 5 because treatment groups are 20%, 20%, 20% and 40%


#
# Randomization happens here
# creating 5 groups: treatment groups 1,2,3 (total = 60%), and control group 4+5 (to be combined after, total = 40%)
# group_1 is the assignment variable to which treatment (3 treatments, 1 control)
treatment <- zh[ , .(idx = sort(unique(idx)), group_1 = cluster_permutation(uniqueN(idx), 5)), keyby = block]

table(treatment$group_1) # it worked, 5 groups each containing 20% of the clusters

# Combine groups 4 & 5 into one group of the control group (40%)
treatment$group_1[treatment$group_1==5] <-4
table(treatment$group_1)
setkey(treatment, idx); setkey(zh, idx)
dim(zh)
zh_tr <- zh[treatment[,.(idx, group_1)]] # merge
dim(zh_tr)

# Create binary variable that assigns treated=1, control=0
zh_tr$treated[zh_tr$group_1==1] <-1
zh_tr$treated[zh_tr$group_1==2] <-1
zh_tr$treated[zh_tr$group_1==3] <-1
zh_tr$treated[zh_tr$group_1==4] <-0

zh_tr[ , mean(group_1), by = idx][,table(V1)] # within each cluster it-s all FALSE or all TRUE. 
zh_tr[ , mean(treated), by = idx][,table(V1)]
zh_tr[ , .(unique(group_1), unique(block)), by = idx][ , mean(V1), by = V2] # within blocks, approx 60% of clusters are treated. Because of value assignment (values range from 1-4), this one is hard to interpret. Look at next one
zh_tr[ , .(unique(treated), unique(block)), by = idx][ , mean(V1), by = V2] # within blocks, approx 60% of clusters are treated
zh_tr[ , t.test(group_1, mu = .5), by = block] # can't reject mean = .5 within blocks without clusters

#####
# Calance tests: overall and within blocks
#####

# What to test?
# 1. gender
# 2. age
# 3. time in Switzerland
# 4. time in Zurich
# 5. cluster size

zh_tr[ , female := as.numeric(gender == "weiblich")]
zh_tr[ , age_d := as.numeric(ymd("2021-09-28") - dob)] # age in days as of today
zh_tr[ , age_y := 2021 - year(dob)] # age in years as of today
zh_tr[ , time_ch_d := as.numeric(ymd("2021-09-28") - arrive_CH)] # time in switzerland in days as of today
zh_tr[ , time_ch_y := 2021 - year(arrive_CH)] # time in switzerland in years as of today
zh_tr[ , time_zh_d := as.numeric(ymd("2021-09-28") - arrive_ZH)]  # time in zurich in days as of today
zh_tr[ , time_zh_y := 2021 - year(arrive_ZH)] # time in zurich in years as of today
zh_tr[ , cluster_size := .N, by = idx]

# Create dummy variables for all treatments
zh_tr$treatment_1[zh_tr$group_1==1] <-1
zh_tr$treatment_1[zh_tr$group_1!=1] <-0

zh_tr$treatment_2[zh_tr$group_1==2] <-1
zh_tr$treatment_2[zh_tr$group_1!=2] <-0

zh_tr$treatment_3[zh_tr$group_1==3] <-1
zh_tr$treatment_3[zh_tr$group_1!=3] <-0

zh_tr$control[zh_tr$group_1==4] <-1
zh_tr$control[zh_tr$group_1!=4] <-0

balance_tester <- function(var, data ){
  return(summary(felm(var  ~ treatment_1 + treatment_2 + treatment_3 | 0 | 0 | idx, data))$coefficients[2:4,4])
}

block_bal_test <- 
  zh_tr[ ,
         lapply(.SD, function(x) summary(felm(x ~ treatment_1 + treatment_2 + treatment_3 |0 | 0 | idx))$coefficients[2:4, 4]),
         .SDcols = c("female", "age_d", "time_ch_d", "time_zh_d", "cluster_size"),
         keyby = block
         ][, .(.SD, "treatment"=rep(c("treatment_1","treatment_2", "treatment_3"),times=5))]


total_bal_test <- 
  zh_tr[ ,
         lapply(.SD, function(x) summary(felm(x ~ treatment_1 + treatment_2 + treatment_3 |0 | 0 | idx))$coefficients[2:4,4]),
         .SDcols = c("female", "age_d", "time_ch_d", "time_zh_d", "cluster_size")
         ][, .(.SD, "treatment"=rep(c("treatment_1","treatment_2", "treatment_3")))]

#testing, by subsetting

block_bal_test_1 <- melt(block_bal_test[treatment=="treatment_1"], ".SD.block"); total_bal_test_1 <- melt(total_bal_test[treatment=="treatment_1"])
block_bal_test_1[, treatment:="treatment_1"]
block_bal_test_2 <- melt(block_bal_test[treatment=="treatment_2"], ".SD.block"); total_bal_test_2 <- melt(total_bal_test[treatment=="treatment_2"])
block_bal_test_2[, treatment:="treatment_2"]
block_bal_test_3 <- melt(block_bal_test[treatment=="treatment_3"], ".SD.block"); total_bal_test_3 <- melt(total_bal_test[treatment=="treatment_3"])# change this
block_bal_test_3[, treatment:="treatment_3"]

total_bal_test_1[ , .SD.block := paste0("Full Sample, N=", zh_tr[ , prettyNum(.N, big.mark = ",")])]
total_bal_test_2[ , .SD.block := paste0("Full Sample, N=", zh_tr[ , prettyNum(.N, big.mark = ",")])]
total_bal_test_3[ , .SD.block := paste0("Full Sample, N=", zh_tr[ , prettyNum(.N, big.mark = ",")])]


bal_tests_1 <- rbind(block_bal_test_1, total_bal_test_1) # Treatment 1
bal_tests_2 <- rbind(block_bal_test_2, total_bal_test_2) # Treatment 2
bal_tests_3 <- rbind(block_bal_test_3, total_bal_test_3) # Treatment 3

# eliminate weird rows in bal_tests
bal_tests_1 <- bal_tests_1[value!="treatment_1"]
bal_tests_2 <- bal_tests_2[value!="treatment_2"]
bal_tests_3 <- bal_tests_3[value!="treatment_3"]

#reconvert value to numeric
bal_tests_1$value <- as.numeric(bal_tests_1$value)
bal_tests_2$value <- as.numeric(bal_tests_2$value)
bal_tests_3$value <- as.numeric(bal_tests_3$value)


## GRAPHS
# Treatment 1
bal_tests_1[ , y := rep(5:1), keyby = .SD.block]
bal_tests_1[ , .SD.block := factor(.SD.block, levels = sort(unique(.SD.block))[c(3,1,2,4,5,6)])]

# checking to see if p-values greater than 0.05 - there is no significant different between treatment and control
bal_plot_1 <- ggplot(data = bal_tests_1, aes(x = value, y = y)) +
  geom_vline(xintercept = c(0)) +
  geom_vline(xintercept = c(.05), linetype = "dashed") +
  geom_vline(xintercept = c(.1), linetype = "dotted") +
  geom_point() +
  scale_y_continuous(breaks = c(5:1), labels = c("Female", "Age", "Time in CH", "Time in ZH", "Cluster Size"), limits = c(0.5, 5.5), expand = c(0,0)) +
  scale_x_continuous(limits = c(-0.005,1.005), expand = c(0,0), breaks = c(0, .05, .1, 1), labels = c("0", "0.05", "0.1", "1")) +
  ylab(NULL) +
  xlab(expression(paste(italic("p"), "-value", sep=""))) +
  facet_wrap( ~ .SD.block, nrow = 2) +
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank()) +
  ggtitle(paste0("Randomization, seed ", seed, "- Treatment 1"))
ggsave(paste0(graphs_path,"bal_", seed, "_treatment1.png"), height = 8.27, width = 13.79)


# Treatment 2
bal_tests_2[ , y := rep(5:1), keyby = .SD.block]
bal_tests_2[ , .SD.block := factor(.SD.block, levels = sort(unique(.SD.block))[c(3,1,2,4,5,6)])]

# checking to see if p-values greater than 0.05 - there is no significant different between treatment and control
bal_plot_2 <- ggplot(data = bal_tests_2, aes(x = value, y = y)) +
  geom_vline(xintercept = c(0)) +
  geom_vline(xintercept = c(.05), linetype = "dashed") +
  geom_vline(xintercept = c(.1), linetype = "dotted") +
  geom_point() +
  scale_y_continuous(breaks = c(5:1), labels = c("Female", "Age", "Time in CH", "Time in ZH", "Cluster Size"), limits = c(0.5, 5.5), expand = c(0,0)) +
  scale_x_continuous(limits = c(-0.005,1.005), expand = c(0,0), breaks = c(0, .05, .1, 1), labels = c("0", "0.05", "0.1", "1")) +
  ylab(NULL) +
  xlab(expression(paste(italic("p"), "-value", sep=""))) +
  facet_wrap( ~ .SD.block, nrow = 2) +
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank()) +
  ggtitle(paste0("Randomization, seed ", seed, "- Treatment 2"))
ggsave(paste0(graphs_path, "bal_", seed, "_treatment2.png"), height = 8.27, width = 13.79)

# Treatment 3
bal_tests_3[ , y := rep(5:1), keyby = .SD.block]
bal_tests_3[ , .SD.block := factor(.SD.block, levels = sort(unique(.SD.block))[c(3,1,2,4,5,6)])]

# checking to see if p-values greater than 0.05 - there is no significant different between treatment and control
bal_plot_3 <- ggplot(data = bal_tests_3, aes(x = value, y = y)) +
  geom_vline(xintercept = c(0)) +
  geom_vline(xintercept = c(.05), linetype = "dashed") +
  geom_vline(xintercept = c(.1), linetype = "dotted") +
  geom_point() +
  scale_y_continuous(breaks = c(5:1), labels = c("Female", "Age", "Time in CH", "Time in ZH", "Cluster Size"), limits = c(0.5, 5.5), expand = c(0,0)) +
  scale_x_continuous(limits = c(-0.005,1.005), expand = c(0,0), breaks = c(0, .05, .1, 1), labels = c("0", "0.05", "0.1", "1")) +
  ylab(NULL) +
  xlab(expression(paste(italic("p"), "-value", sep=""))) +
  facet_wrap( ~ .SD.block, nrow = 2) +
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank()) +
  ggtitle(paste0("Randomization, seed ", seed, "- Treatment 3"))
ggsave(paste0(graphs_path, "bal_", seed, "_treatment3.png"), height = 8.27, width = 13.79)



## Export list of groups
treatment1 <- zh_tr[group_1 == 1 , .(PID, last_name, given_name, first_name, name_supplement, street, house_number, PLZ, city, gender, dob, nat, arrive_CH, arrive_ZH)]
treatment2 <- zh_tr[group_1 == 2 , .(PID, last_name, given_name, first_name, name_supplement, street, house_number, PLZ, city, gender, dob, nat, arrive_CH, arrive_ZH)]
treatment3 <- zh_tr[group_1 == 3 , .(PID, last_name, given_name, first_name, name_supplement, street, house_number, PLZ, city, gender, dob, nat, arrive_CH, arrive_ZH)]
control <- zh_tr[group_1 == 4 , .(PID, last_name, given_name, first_name, name_supplement, street, house_number, PLZ, city, gender, dob, nat, arrive_CH, arrive_ZH)]

# Some housekeeping
treatment1[ , arrive_CH := as.character(format(arrive_CH, "%d.%m.%Y"))]
treatment1[ , arrive_ZH := as.character(format(arrive_ZH, "%d.%m.%Y"))]
treatment1[ , dob := as.character(format(dob, "%d.%m.%Y"))]

treatment2[ , arrive_CH := as.character(format(arrive_CH, "%d.%m.%Y"))]
treatment2[ , arrive_ZH := as.character(format(arrive_ZH, "%d.%m.%Y"))]
treatment2[ , dob := as.character(format(dob, "%d.%m.%Y"))]

treatment3[ , arrive_CH := as.character(format(arrive_CH, "%d.%m.%Y"))]
treatment3[ , arrive_ZH := as.character(format(arrive_ZH, "%d.%m.%Y"))]
treatment3[ , dob := as.character(format(dob, "%d.%m.%Y"))]

control[ , arrive_CH := as.character(format(arrive_CH, "%d.%m.%Y"))]
control[ , arrive_ZH := as.character(format(arrive_ZH, "%d.%m.%Y"))]
control[ , dob := as.character(format(dob, "%d.%m.%Y"))]


setkey(treatment1, PID)
treatment1

setkey(treatment2, PID)
treatment2

setkey(treatment3, PID)
treatment3

setkey(control, PID)
control

list.files()
fwrite(treatment1, paste0(data_path,"randomized_allocation_Complexity.csv"))     # Treatment "complexity barrier" (on a red flyer)
# NB: File above was previously saved as: fwrite(treatment1, "flyer_rot.csv")   

fwrite(treatment2, paste0(data_path,"randomized_allocation_Requirements.csv"))    # Treatment "knowledge barrier" (on a blue flyer)
# NB: File above was previously saved as: fwrite(treatment2, "flyer_blau.csv")   

fwrite(treatment3, paste0(data_path, "randomized_allocation_Welcome.csv"))         # Treatment "welcome" (the letter from the City of Zurich)
# NB: File above was previously saved as: fwrite(treatment3, "brief.csv")        

fwrite(control, paste0(data_path,"randomized_allocation_Control.csv"))    # Control group in Wave 1 (Treated in Wave 2)
# NB: File above was previously saved as: fwrite(control, "control_group.csv")  


fwrite(zh_tr, paste0(data_path,"FullSample_withRandom.csv")) # encoding is UTF-8
