#####

rm(list=ls())

## Set working directories
data_raw_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data_raw/")
data_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data/")

# Load packages
library("data.table")
library("readxl")
library("lubridate")
library("stringr")
library("dplyr")

# read in the October 11 2021 to 31 March 2022 data and convert to data.table
# Note: the data is badly named by the Canton, the dates are actually 11.10.2021-31.03.2022
apps <- read_xlsx(paste0(data_raw_path,"01-Gesuchseingaenge_09.10.2021_31.01.2022 Zuerich.xlsx"))


setDT(apps)

# create easier to use column names
colnames(apps)
replace_names <- c(
  "FIRMA" = "company",               # takes only one value
  "GGRP" = "ggrp",                   # takes only one value
  "GESCH_JAHR" = "app_year",
  "GESCH_NR" = "app_nr",
  "EINGANG" = "app_date",
  "NAME" = "last_name",
  "VORNAME" = "first_name",
  "GEBURTSDATUM" = "dob",
  "GESCHLECHT" = "gender",
  "ZIVILSTAND" = "marital",
  "PARTEIART" = "party_type" ,         # BEW=applicant, KND=child, EHP=spouse
  "STRASSE" = "street_house_address",
  "ZUSATZ" = "zusatz",
  "PLZ" = "PLZ",
  "ORT" = "city",
  "LAND_CODE" = "nat_code",
  "NATION" = "nat",
  "PARTEIART_GEM" = "party_type_gem",  # takes only one value
  "GEM_NR" = "mun_nr" ,                # takes only one value
  "GEMEINDE" = "municipality"          # takes only one value
)

setnames(apps, old = names(replace_names), new = replace_names)

# Change country names from the application data to match the administrative data
corrections <- c(
"Aegypten"	=	"Ägypten",
"Aethiopien"	=	"Äthiopien",
"Bangladesh"	=	"Bangladesch",
"Bosnien und Herzegowina"	=	"Bosnien-Herz.",
"China"	=	"China, Volksr.",
"Dominikanische Republik"	=	"Dominik. Rep.",
"Moldova"	=	"Moldau",
"Österreich"	=	"Österreich",
"Serbien und Montenegro"	=	"Serbien",
"Tschechische Republik"	=	"Tschechische R.",
"Vereinigte Staaten"	=	"USA"
)


apps[nat %chin% names(corrections), nat := corrections[nat]]

# Reformat the date variables
apps$dob <- as.Date(apps$dob, "CET")      # date was in POSIXct format

for(j in c("app_date")) set(apps, j = j, value = as.Date(apps[[j]]))


# Drop people that were not 18 by mid 2021 (hence, not in the study)
apps_young <- apps[dob > ymd("2003-06-30")]    # keep just because
apps <- apps[dob <= ymd("2003-06-30")]


# correct streets that are in the form "XXstr." to "XXstrasse" / "XXStr." to "XXStrasse"
apps$street_house_address<- str_replace(apps$street_house_address, "str. ", "strasse ")
apps$street_house_address<- str_replace(apps$street_house_address, "Str. ", "Strasse ")

# create some simple versions of key variables
for(j in c("first_name", "last_name", "street_house_address")){
  set(apps, j = paste0(j, "_S"), value = tolower(stringi::stri_trans_general(apps[[j]], "latin-ascii")))
}

# Change gender to match admin data: from M to männlich and from F to weiblich
apps$gender<- str_replace(apps$gender, "M", "männlich")
apps$gender<- str_replace(apps$gender, "F", "weiblich")




#----------
# Begin matching with admin data

#read in the data from the admin 
admin <- fread(paste0(data_path,
  "FullSample_withRandom.csv"),
  colClasses = "character"
)


admin$dob <- as.Date(admin$dob, format = "%Y-%m-%d")


# Create some simple versions of key variables, to match apps dataframe
for( j in c("first_name_S", "last_name_S")) set(admin, j = j, value = tolower(admin[[j]]))
set(admin, j = "street_house_address", value = tolower(stringi::stri_trans_general(admin$street_house_address, "latin-ascii")))

# Easy try: first and last names (accent stripped & all lower-case versions)
apps[ , matched_1 := paste0(first_name_S, " ", last_name_S) %chin% admin[,paste0(first_name_S, " ", last_name_S)]]
apps[ , .N, matched_1] # gets about 25% of the people (315/1214)

# look for matches based on name distance & dob
test <- adist(
  x = apps[ , paste0(last_name_S, ", ", first_name_S, ", ", dob)],
  y = admin[ , paste0(last_name_S, ", ", first_name_S, ", ", dob)]
)

dtf <- as.data.table(test)
best_matches <- apply(dtf, 1, function(x) which(x == min(x), useNames = F))
match_quality <- apply(dtf, 1, function(x) min(x))
reps <- sapply(best_matches, length)
apps_cols <- unlist(mapply(rep, x = 1:apps[,.N], times = reps, SIMPLIFY = T))
match_quals <- unlist(mapply(rep, x = match_quality, times = reps, SIMPLIFY = T))

# Create indices for matching
apps[ , index := .I]
admin[ , index := .I]

multi_match <-
  cbind(
    apps[apps_cols, .(
      "f_name_app" = first_name_S,
      "l_name_app" = last_name_S,
      "dob_app" = dob,
      "nat_app" = nat,
      "gender_app" = gender,
      "add_app" = street_house_address,
      # "arrive_ZH_app" = arrive_ZH,
      # "arrive_CH_app" = arrive_CH,
      "match_names" = matched_1,
      # "match_name_dob" = matched_2,
      "match_quals" = match_quals,
      "index_app" = index#,
      # "letter_1_app" = letter_1,
      # "letter_2_app" = letter_2
    )],
    admin[unlist(best_matches), .(
      "f_name_adm" = first_name_S,
      "l_name_adm" = last_name_S,
      "dob_adm" = dob,
      "nat_adm" = nat,
      "gender_adm" = gender,
      "add_adm" = street_house_address,
      # "arrive_ZH_adm" = arrive_ZH,
      # "arrive_CH_adm" = arrive_CH,
      "index_adm" = PID
    )]
  )


# Create distance variable between two datasets based on birthdate
multi_match[ , dob_dist := abs(dob_app - dob_adm)] 


# Create dummy variable =1 if nationality is the same
multi_match[!is.na(nat_app) , nat_match := nat_app == nat_adm ] 

# Aggregate the number of matches
multi_match[ , N_matches := uniqueN(index_adm), by = .(index_app)]

# Create dummy variable =1 if gender is the same
multi_match[ , gender_match := gender_app == gender_adm]


setcolorder(multi_match, neworder = c("index_app", "f_name_app", "f_name_adm", "l_name_app", "l_name_adm", "dob_app", "dob_adm", "dob_dist", "match_quals", "nat_app",  "nat_adm", "gender_app", "gender_adm", "add_app", "add_adm"))
setkey(multi_match, N_matches, match_quals, index_app)


# Cases:
# If name and birthdate match perfectly, accept. 
# If only slight name/birthdate differences but matching addresses, accept.
# I checked this by looking manually - among those not yet confirmed (after Step 1 below), sort by lowest dob_dist, and check manually at names (sometimes middle names were different or not reported in one of the two datasets) and then check addresses
# potential exclusion: if there are some close matches with small address differences

set(multi_match, j = "confirmed_match", value = FALSE)
multi_match[match_quals %in% c(0,1,2, 4, 5), confirmed_match := T] # Step 1
multi_match[index_app %in% c(450, 704, 1107, 479, 1031, 1065, 1067,567,623,896,1043,643,1194), confirmed_match := T] # Step 2
multi_match[dob_dist %in% c(6935,1675,912), confirmed_match := F] # These don't match


# Set up the merge of correct matches
dim(apps)
apps_merge <- merge(apps, multi_match[confirmed_match == T, .(index_app, index_adm)], all.x = T, all.y = F, by.x = "index", by.y = "index_app")
setnames(apps_merge, old = "index", new = "index_app")
dim(apps_merge)

# remove some of the helper columns before saving the processed file for merging
for(j in c("first_name_S", "last_name_S", "street_house_address_S", "matched_1", "index_app")) set(apps_merge, j = j, value = NULL)

# create variable "applied" indicating whether the individual applied (and is in our original sample) or not
apps_merge[!is.na(index_adm), applied:=1]
apps_merge[is.na(index_adm), applied:=0]

# Save a file of those who applied between October 2021 and March 2022, with an indicator=1 if they were part of our sample
fwrite(apps_merge, paste0(data_path, "app_oct_mar_2022.csv"))


#---------- Prepare admin data

# Merge those who applied back onto the original admin data
admin_merge <- merge(admin, apps_merge, all.x = T, all.y = F, by.x = "PID", by.y = "index_adm")
admin_merge[ , dob_dist := abs(lubridate::ymd(dob.x) - lubridate::ymd(dob.y))]

# remove some of the helper columns before saving the processed file for merging
admin_merge = subset(admin_merge, select = -c(last_name.y,
                                              first_name.y,
                                              dob.y, gender.y, 
                                              company, ggrp, gender.y,
                                              marital, party_type, 
                                              street_house_address.y, 
                                              zusatz, PLZ.y, city.y,
                                              nat_code, nat.y, party_type_gem,
                                              mun_nr) )

# 1. Assign nationality groups to individuals, same method as the blocking variable "block"
DE <- "DE" 
dev_dem <- c("AT", "GB", "FR", "FI", "LI", "SE", "NL", "IE", "BE", "DK", "NO", "LU", "IS", "US", "CA", "AU", "NZ")
EU_south <- c("IT", "ES", "PT", "GR", "CY")
EU_east <- c("AL", "SI", "MK", "HR", "XKO", "RS", "BA", "ME", "PL", "RO", "SK", "RU", "BG", "BY", "HU", "EE", "UA", "LT", "LV", "MD", "CZ")
rest_of_world <- c("IQ", "TR", "IR", "MA", "LB", "TN", "SD", "IL", "EG", "JO", "AF", "ZZX", "NA", "DZ", "SY", "SA", "LY", "AZ", "YE", "PS", "GE", "BD", "CN", "IN", "ID", "JP", "KH", "KZ", "KG", "KR", "MY", "MN", "NP", "PK", "PH", "SG", "LK", "TW", "TH", "UZ", "VN", "AO", "ET", "CI", "ER", "GM", "GH", "CM", "KE", "CD", "CG", "ML", "MU", "NG", "SN", "ZW", "SO", "ZA", "TZ", "TG", "CO", "BR", "JM", "AR", "DO", "BO", "CL", "PY", "CU", "PA", "MX", "VE", "GT", "EC", "HT", "CR", "UY", "PE", "NI", "MG", "HN", "MM")

# Assign to blocks
admin_merge[ iso_code %in% DE, block_ind := paste0("Germany, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ iso_code %in% dev_dem, block_ind := paste0("Developed Democracies, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ iso_code %in% EU_south, block_ind := paste0("Southern Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ iso_code %in% EU_east, block_ind := paste0("Central and Eastern Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ iso_code %in% rest_of_world, block_ind := paste0("Rest of World, N=", prettyNum(.N, big.mark = ","))]


## 2. Regional Groups, type 1 (region1)
# Americas & Caribbean
AC <- c("Argentinien","Bolivien","Brasilien","Chile","Costa Rica","Dominik. Rep.","Ecuador","Guatemala","Honduras","Jamaika","Kanada","Kolumbien","Kuba","Mexiko","Nicaragua","Panama","Paraguay","Peru","Uruguay","USA","Venezuela")
# Central Asia
CA <- c("Aserbaidschan","Belarus","Georgien","Kasachstan","Russland","Tschechische R.","Usbekistan")
# Central-East Europe
CEE<- c("Estland","Lettland","Litauen","Moldau","Polen","Slowakei","Ukraine","Ungarn")
# East Asia and Pacific
EAP <- c("Australien","China, Volksr.","Indonesien","Japan","Kambodscha","Korea (Süd-)","Malaysia","Mongolei","Myanmar","Neuseeland","Philippinen","Singapur","Taiwan","Thailand","Vietnam")
# Spain and Portugal
ES_PT <- c("Portugal","Spanien")
# Germany and Austria
GE_AT <- c("Österreich","Deutschland")
# Italy
IT <- "Italien"
# Middle East and Northern Africa
MENA <- c("Ägypten","Algerien","Irak","Iran","Israel","Jemen","Jordanien","Libanon","Libyen","Marokko","Palästina","Saudi-Arabien","Syrien","Tunesien") 
# South Asia
SA <- c("Afghanistan","Bangladesch","Indien","Nepal","Pakistan","Sri Lanka")
# South-East Europe
SEE <- c("Albanien","Bosnien-Herz.","Bulgarien","Kosovo","Kroatien","Montenegro","Nordmazedonien","Rumänien","Serbien","Slowenien","Türkei","Griechenland")
# Sub-Saharian Africa
SSA <- c("Angola","Äthiopien","Eritrea","Gambia","Ghana","Kamerun","Kenia","Kongo, Dem. Rep","Madagaskar","Nigeria","Senegal","Simbabwe","Somalia","Südafrika","Sudan","Tansania","Togo")
# Western Europe
WE <- c("Norwegen","Belgien","Dänemark","Finnland","Frankreich","Grossbritannien","Irland","Island","Liechtenstein","Luxemburg","Niederlande","Schweden")

Stateless <- "Staatenlos"


# problem with Dominik. Rep., Sri Lanka, China, Tschechinsce R., Kongo Dem Rep, Korea Sud, Costa Rica
# use iso codes for them
AC_2 <- c("DO", "CR")
SA_2 <- "LK"
EAP_2 <- c("CN", "KR")
CA_2 <- "CZ"
SSA_2 <- "CD"


# create variable region1
admin_merge[ nat.x %in% AC | iso_code %in% AC_2 , region1 := paste0("Americas & Caribbean, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% CA | iso_code %in% CA_2, region1 := paste0("Central Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% CEE, region1 := paste0("Central-East Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% EAP | iso_code %in% EAP_2, region1 := paste0("East Asia & Pacific, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% ES_PT, region1 := paste0("Spain and Portugal, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% GE_AT, region1 := paste0("Germany and Austria, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% IT, region1 := paste0("Italy, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% MENA, region1 := paste0("Middle Easy and Northern Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SA | iso_code %in% SA_2, region1 := paste0("South Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SEE, region1 := paste0("South-East Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SSA | iso_code %in% SSA_2, region1 := paste0("Sub-Saharian Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% WE, region1 := paste0("Western Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% Stateless, region1 := paste0("Stateless, N=", prettyNum(.N, big.mark = ","))]




## 3. Regional Groups, type 2 (region2)
# Americas & Caribbean
AC <- c("Argentinien","Bolivien","Brasilien","Chile","Costa Rica","Dominik. Rep.","Ecuador","Guatemala","Honduras","Jamaika","Kanada","Kolumbien","Kuba","Mexiko","Nicaragua","Panama","Paraguay","Peru","Uruguay","USA","Venezuela")

# East Asia and Pacific
EAP <- c("Australien","China, Volksr.","Indonesien","Japan","Kambodscha","Korea (Süd-)","Malaysia","Mongolei","Myanmar","Neuseeland","Philippinen","Singapur","Taiwan","Thailand","Vietnam")

# Germany and Austria
GE_AT <- c("Österreich","Deutschland")

# Middle East and Northern Africa
MENA <- c("Ägypten","Algerien","Irak","Iran","Israel","Jemen","Jordanien","Libanon","Libyen","Marokko","Palästina","Saudi-Arabien","Syrien","Tunesien") 

# Stateless
Stateless <- "Staatenlos"

# South Asia
SA <- c("Afghanistan","Bangladesch","Indien","Nepal","Pakistan","Sri Lanka")

# Sub-Saharian Africa
SSA <- c("Angola","Äthiopien","Eritrea","Gambia","Ghana","Kamerun","Kenia","Kongo, Dem. Rep","Madagaskar","Nigeria","Senegal","Simbabwe","Somalia","Südafrika","Sudan","Tansania","Togo")

# Western Europe
WE <- c("Portugal","Spanien","Italien","Griechenland","Norwegen","Belgien","Dänemark","Finnland","Frankreich","Grossbritannien","Irland","Island","Liechtenstein","Luxemburg","Niederlande","Schweden")

# Eastern Europe and Central Asia
EECA <- c("Aserbaidschan","Belarus","Georgien","Kasachstan","Russland","Tschechische R.","Usbekistan","Estland","Lettland","Litauen","Moldau","Polen","Slowakei","Ukraine","Ungarn","Albanien","Bosnien-Herz.","Bulgarien","Kosovo","Kroatien","Montenegro","Nordmazedonien","Rumänien","Serbien","Slowenien","Türkei")


# problem with Dominik. Rep., Sri Lanka, China, Tschechinsce R., Kongo Dem Rep, Korea Sud, Costa Rica
# use iso codes for them
AC_2 <- c("DO", "CR")
SA_2 <- "LK"
EAP_2 <- c("CN", "KR")
CA_2 <- "CZ"  # goes in EECA
SSA_2 <- "CD"


# create variable region1
# REGIONS: SA, MENA, EECA, EAP, AC, SSA, WE, GE_AT, Stateless
admin_merge[ nat.x %in% AC | iso_code %in% AC_2 , region2 := paste0("Americas & Caribbean, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% EECA | iso_code %in% CA_2, region2 := paste0("Central Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% EAP | iso_code %in% EAP_2, region2 := paste0("East Asia & Pacific, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% GE_AT, region2 := paste0("Germany and Austria, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% MENA, region2 := paste0("Middle Easy and Northern Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SA | iso_code %in% SA_2, region2 := paste0("South Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SSA | iso_code %in% SSA_2, region2 := paste0("Sub-Saharian Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% WE, region2 := paste0("Western Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% Stateless, region2 := paste0("Stateless, N=", prettyNum(.N, big.mark = ","))]




## 3. Regional Groups, type 3 (region3)
# Americas & Caribbean
AC <- c("Argentinien","Bolivien","Brasilien","Chile","Costa Rica","Dominik. Rep.","Ecuador","Guatemala","Honduras","Jamaika","Kanada","Kolumbien","Kuba","Mexiko","Nicaragua","Panama","Paraguay","Peru","Uruguay","USA","Venezuela")
# Central Asia
CA <- c("Aserbaidschan","Belarus","Georgien","Kasachstan","Russland","Tschechische R.","Usbekistan")
# Central-East Europe
CEE<- c("Estland","Lettland","Litauen","Moldau","Polen","Slowakei","Ukraine","Ungarn")
# East Asia and Pacific
EAP <- c("Australien","China, Volksr.","Indonesien","Japan","Kambodscha","Korea (Süd-)","Malaysia","Mongolei","Myanmar","Neuseeland","Philippinen","Singapur","Taiwan","Thailand","Vietnam")
# Spain and Portugal
ES <- "Spanien"
PT <- "Portugal"
# Germany and Austria
GE <- "Deutschland"
AT <- "Österreich"
# Italy
IT <- "Italien"
# Middle East and Northern Africa
MENA <- c("Ägypten","Algerien","Irak","Iran","Israel","Jemen","Jordanien","Libanon","Libyen","Marokko","Palästina","Saudi-Arabien","Syrien","Tunesien") 
# South Asia
SA <- c("Afghanistan","Bangladesch","Indien","Nepal","Pakistan","Sri Lanka")
# South-East Europe
SEE <- c("Albanien","Bosnien-Herz.","Bulgarien","Kroatien","Montenegro","Nordmazedonien","Rumänien","Slowenien","Griechenland")
RS <- "Serbien"
XKO <- "Kosovo"
TR <- "Türkei"
# Sub-Saharian Africa
SSA <- c("Angola","Äthiopien","Eritrea","Gambia","Ghana","Kamerun","Kenia","Kongo, Dem. Rep","Madagaskar","Nigeria","Senegal","Simbabwe","Somalia","Südafrika","Sudan","Tansania","Togo")
# Western Europe
WE <- c("Norwegen","Belgien","Dänemark","Finnland","Irland","Island","Liechtenstein","Luxemburg","Niederlande","Schweden")
UK <- "Grossbritannien"
FR <- "Frankreich"
Stateless <- "Staatenlos"


# problem with Dominik. Rep., Sri Lanka, China, Tschechinsce R., Kongo Dem Rep, Korea Sud, Costa Rica
# use iso codes for them
AC_2 <- c("DO", "CR")
SA_2 <- "LK"
EAP_2 <- c("CN", "KR")
CA_2 <- "CZ"
SSA_2 <- "CD"




admin_merge[ nat.x %in% AC | iso_code %in% AC_2 , region3 := paste0("Americas & Caribbean, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% CA | iso_code %in% CA_2, region3 := paste0("Central Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% CEE, region3 := paste0("Central-East Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% EAP | iso_code %in% EAP_2, region3 := paste0("East Asia & Pacific, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% ES, region3 := paste0("Spain, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% PT, region3 := paste0("Portugal, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% GE, region3 := paste0("Germany, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% AT, region3 := paste0("Austria, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% IT, region3 := paste0("Italy, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% FR, region3 := paste0("France, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% UK, region3 := paste0("UK, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% RS, region3 := paste0("Serbia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% XKO, region3 := paste0("Kosovo, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% TR, region3 := paste0("Turkey, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% MENA, region3 := paste0("Middle Easy and Northern Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SA | iso_code %in% SA_2, region3 := paste0("South Asia, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SEE, region3 := paste0("South-East Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% SSA | iso_code %in% SSA_2, region3 := paste0("Sub-Saharian Africa, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% WE, region3 := paste0("Western Europe, N=", prettyNum(.N, big.mark = ","))]
admin_merge[ nat.x %in% Stateless, region3 := paste0("Stateless, N=", prettyNum(.N, big.mark = ","))]


# Save
fwrite(admin_merge, paste0(data_path,"FullSample_withRandom_applied.csv"))



#### ADD IN THOSE WHO MOVED DURING WAVE 1
# The City send us "cleaned lists" by treatment, of those who DID NOT move during the treatment period of Wave 1. We want to add a column to indicate that they have moved.

# Read in the cleaned lists sent by the City
welcome_cleaned <- readxl::read_excel(paste0(data_raw_path, "Brief 2. Versand_versandbereit April 2022.xlsx"))
complexity_cleaned <- readxl::read_excel(paste0(data_raw_path, "Flyer rot 2. Versand_versandbereit April 2022.xlsx"))
requirements_cleaned <- readxl::read_excel(paste0(data_raw_path, "Flyer blau 2. Versand_versandbereit April 2022.xlsx" ))
control_cleaned <- readxl::read_excel(paste0(data_raw_path, "control_group_überprüft.xlsx"))

# We merge the final dataset (admin_merge) with this
# Only the control_cleaned dataset has PID, so we have to merge using all other information.
# First, we add in a PID column
welcome_cleaned <- welcome_cleaned %>% mutate(PID=999)
complexity_cleaned <- complexity_cleaned %>% mutate(PID=999)
requirements_cleaned <- requirements_cleaned %>% mutate(PID=999)

# Append
df_cleaned <- rbind(welcome_cleaned, complexity_cleaned, requirements_cleaned, control_cleaned)

# Add in column MOVERS=0 to indicate that they did not move
# Change date format
df_cleaned <- df_cleaned %>% mutate(movers=0) 
df_cleaned$dob <- as.Date(df_cleaned$dob, format = "%d.%m.%Y")
df_cleaned$arrive_CH <- as.Date(df_cleaned$arrive_CH, format = "%d.%m.%Y")
df_cleaned$arrive_ZH <- as.Date(df_cleaned$arrive_ZH, format = "%d.%m.%Y")
df_cleaned$PLZ <- as.character(df_cleaned$PLZ)

admin_merge$arrive_CH <- as.Date(admin_merge$arrive_CH, format = "%Y-%m-%d")
admin_merge$arrive_ZH <- as.Date(admin_merge$arrive_ZH, format = "%Y-%m-%d")

# Merge
df_movers <- tidylog::left_join(admin_merge, df_cleaned, by=c('last_name.x'='last_name',
                                                     'first_name.x'='first_name', 
                                                     'street', 'house_number',
                                                     'PLZ.x'='PLZ', 
                                                     'city.x'='city', 
                                                     'gender.x'='gender', 
                                                     'dob.x'='dob',
                                                     'nat.x'='nat',
                                                     'arrive_CH',
                                                     'arrive_ZH'))

# Set the movers to 1 and remove the old PID (was just to check)
df_movers$movers <- ifelse(is.na(df_movers$movers), 1, df_movers$movers) 
df_movers <- df_movers %>% select(-PID.y)

# Clean before saving
df_movers <- df_movers  %>% 
  rename('PID'='PID.x',
          'dob'='dob.x',
         'first_name' = 'first_name.x',
         'last_name' = 'last_name.x',
         'city' = 'city.x',
         'gender' = 'gender.x',
         'nat' = 'nat.x',
         'PLZ' = 'PLZ.x',
         'street_house_address' = 'street_house_address.x',
         'given_name'='given_name.x',
         'name_supplement'='name_supplement.x')
df_movers <- df_movers  %>% dplyr::select(-ends_with(".y"))

# Save
write.csv(df_movers, paste0(data_path,"FullSample_withRandom_applied_movers.csv"))

