# allocate optimal treatment

rm(list=ls())

setwd("/Users/kahrens/MyProjects/policy_tree")

library("tidyverse")
library("readr")
library("policytree")
library("grf")
library("glmnet")
library("glmnetUtils")
library(DiagrammeR)
library("DiagrammeRsvg")
library("foreach")
library("parallel")
library("readxl")
library("randomizr")
library("fixest")
library("xtable")

sessionInfo()
source("./Code/auxiliary_functions.R")


data_folder <- "/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data_raw"
out_folder <- "/Users/kahrens/MyProjects/policy_tree/Outputs/"
models_folder <- "/Users/kahrens/MyProjects/policy_tree/Models/"
data_processed_folder <- "/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data"
#out_folder <- "/Users/kahrens/MyProjects/policy_tree_paper/"


###############################################################
### data import and preparation                             ###
###############################################################

dta <- read_csv(paste0(data_processed_folder ,"/FullSample_withRandom_applied_movers.csv"))
regions <- read_excel(paste0(data_folder,
                             "/region_list.xlsx"))

# exclude movers
dta <- dta %>% 
  filter(movers==0)

# only relevant columns
dta <- dta %>%
  dplyr::select(PID,iso_code,treated,gender,dob,arrive_CH,arrive_ZH,nat,
                treatment_1,treatment_2,treatment_3,applied,idx,movers
  )

dta <- left_join(dta,regions %>% dplyr::select(-nat),"iso_code")

dta <- dta %>% 
  mutate(age = 2022 - lubridate::year(dob),
         time_in_ch = 2022 - lubridate::year(arrive_CH),
         time_in_zh = 2022 - lubridate::year(arrive_ZH))

# outcome: applied
dta <- dta %>% 
  mutate(applied = case_when(applied==1~1,
                             applied==0~0,
                             is.na(applied)~0))

# first test OLS
summary(lm(applied~treatment_1+treatment_2+treatment_3,data=dta))

dta <- dta %>% 
  mutate(
    treat_type = treatment_1*1+treatment_2*2+treatment_3*3,
    gender.x=as.factor(gender),
    nat.x = as.factor(nat),
    region1=as.factor(region1),
    region2=as.factor(region2)
  ) 

## first and 2nd wave indicator
# note that the wave1 indicator is TRUE for the whole sample
dta <- dta %>% 
  mutate(wave1 = TRUE,
         wave2 = treated==0 & applied == 0)
wave1 <- dta$wave1
wave2 <- dta$wave2
n_wave2 <- sum(wave2)

#write_csv(dta,paste0(data_processed_folder,"/FullSample_withRandom_applied_movers_processed.csv"))



###############################################################
### randomisation for 2nd wave:                             ###
### 1/3 policy rule, 1/3 always best treatment, 1/3 control ###
###############################################################

set.seed(1942)
wave2_random <- dta %>%
  filter(wave2) %>% 
  dplyr::select(PID,idx) %>%
  arrange(idx) %>%
  mutate(wave2_arm = cluster_ra(idx,prob_each=c(1/2,1/6,1/6,1/6),
                                conditions=c("Policy rule","Treatment 1","Treatment 2","Treatment 3"))) 

# check that it worked: variance within clusters should be nil
wave2_random %>%
  mutate(wave2_arm_id = as.numeric(wave2_arm)) %>%
  group_by(idx) %>%
  mutate(wave2_var = var(wave2_arm_id)) %>%
  pull(wave2_var) %>%
  table(.,useNA="al")
# allocation across treatment arms
table(wave2_random$wave2_arm,useNA="al")
dta <- left_join(dta,wave2_random %>% dplyr::select(-idx),c("PID"))
dta$wave2_arm %>% table()


###############################################################
### descriptives                                            ###
###############################################################

library("fastDummies")

dta |>
  mutate(female=1*(gender=="weiblich")) |>
  dummy_cols(select_columns = "region4") |>
  select(time_in_ch,time_in_zh,age,female,applied,starts_with("region4_")) |>
  tidyr::pivot_longer(-0) |> 
  mutate(name=case_when(name=="time_in_ch"~"Years in Switzerland",
                        name=="time_in_zh"~"Years in Zurich",
                        name=="age"~"Age",
                        name=="female"~"Female",
                        name=="applied"~"Naturalization application",
                        TRUE~name
  ),
  name=str_replace(name,"region4\\_",""),
  name=str_replace(name,"\\&","&")
  ) |>
  group_by(name) |>
  summarise(avg= mean(value),
            sd = sd(value),
            min=min(value),
            max=max(value),
            N=sum(!is.na(value))
  ) |>
  mutate(sort=case_when(name=="Age"~1,
                        name=="Years in Switzerland"~1,
                        name=="Years in Zurich"~1,
                        name=="Female"~1,
                        name=="Naturalization application"~1,
                        TRUE~9
  )) |>
  arrange(sort,name) |>
  select(-sort) |>
  mutate(empty="") |>
  select(empty,everything()) |> 
  xtable() |>
  print(include.rownames=F,
        file=paste0(out_folder,"tables/descriptives_wave1.tex")
        )

dta$nat |> table() |> prop.table() |> sort(decreasing=T) -> ctries
ctries[1:10]

###############################################################
### create Y, W, X matrices                                 ###
###############################################################

X1 <- model.matrix(~region4+age+time_in_ch+time_in_zh+gender,data=dta)
rownames(X1) <- dta$PID
W1 <- as.factor(dta[wave1,]$treat_type)
Y1 <- dta[wave1,]$applied
X2 <- X1[wave2,]
Y <- dta$applied
W <- as.factor(dta$treat_type)


save(X1,Y1,W1,X2,Y,W,
     file=paste0(data_processed_folder,"/data_matrices.RData"))



###############################################################
### derive optimal treatments                               ###
###############################################################

# brief glimpse on treatment effect     
summary(lm(Y1~W1))
summary(lm(applied~treatment_1+treatment_2+treatment_3,data=dta))
        
set.seed(563)

prop.table(table(W1))
forest.all <- grf::multi_arm_causal_forest(X1,Y1,W1,W.hat =c(2/5,1/5,1/5,1/5)) 
forest.all
 
# Compute doubly robust reward estimates.
Gamma.matrix <- double_robust_scores(forest.all)
if (FALSE) {
  pol_fit <- policy_tree(X1,Gamma.matrix,3)
  save(pol_fit,file=paste0(models_folder,"policytree_d3.RData"))
} else {
  load(file=paste0(models_folder,"policytree_d3.RData"))
}
predict(pol_fit,newdata=X2) %>% table() # reported in text
### used in Figure 2
pol_fit
predict(pol_fit,newdata=X1,type="node.id") %>% table()  
predict(pol_fit,newdata=X2,type="node.id") %>% table()

W_best <- predict(pol_fit,newdata=X2)-1
table(W_best)

predict(pol_fit,newdata=X2,type="node.id") %>% table()
pol_fit

# plot
library("DiagrammeRsvg")
plot(pol_fit,leaf.labels=c("No treatment","Complexity","Requirements","Welcome"))
tree.plot=plot(pol_fit,leaf.labels=c("No treatment","Complexity","Requirements","Welcome"))
cat(DiagrammeRsvg::export_svg(tree.plot), file = paste0(out_folder,"figures/policy_tree_R.svg"))
 
## average treatment effect first wave
tau.hat <- average_treatment_effect(forest.all)
tau.hat

###############################################################
### heterogeneity analysis                                  ###
###############################################################

# Compare regions with high and low estimated CATEs
high_effect =forest.all$predictions > apply(forest.all$predictions,2,median)

j <- 3
high_effect_j<-high_effect[,j]
ate.high = average_treatment_effect(forest.all, subset = high_effect_j)
ate.low = average_treatment_effect(forest.all, subset = !high_effect_j)
paste("95% CI for difference in ATE:",
      round(ate.high[j,1] - ate.low[j,1], 3), "+/-",
      round(qnorm(0.975) * sqrt(ate.high[j,2]^2 + ate.low[j,2]^2), 3))

### heterogeneity by sex #####################################################
gender_ate <- list()
for (j in 0:1) { 
  ate.j = average_treatment_effect(forest.all, subset = j==X1[,"genderweiblich"])
  gender_ate[[paste(j)]] <- ate.j |> mutate(gender=j) |>
    mutate(gender=if_else(gender==0,"male","female")
    )
}
gender_ate <- bind_rows(gender_ate)

p4 <- gender_ate |>
  bind_rows(tau.hat |> mutate(gender="All"))  |>
  mutate(contrast=case_when(contrast=="1 - 0"~"Complexity",
                            contrast=="2 - 0"~"Requirements",
                            contrast=="3 - 0"~"Welcome",
  )) |>
  mutate(gender=fct_relevel(gender,"All")) |> 
  ggplot() +
  geom_pointrange(aes(x=gender,y=estimate,ymin=estimate-1.96*std.err,
                      ymax=estimate+1.96*std.err,col=contrast,
                      linetype=contrast,
                      shape=contrast),
                  linewidth=1.0,size=.7,
                  position=position_dodge(width = .5)) +
  geom_hline(yintercept=0,alpha=.3)+
  theme_classic() +
  scale_color_discrete("Treatment letter")+
  scale_shape_discrete("Treatment letter")+
  scale_linetype_discrete("Treatment letter")+
  theme(#axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
    text=element_text(size=14))+
  xlab("Gender") + ylab("Effect of treatment letter on application rate")
ggsave(paste0(out_folder,"figures/hetero_gender.png"),width=18,height=15,unit="cm")


### heterogeneity by region #####################################################
reg_list <- c("region4Asia" , "region4Central-East Europe" ,"region4Germany and Austria" ,           
 "region4Italy"   ,                        "region4Middle Easy and Northern Africa" ,"region4South-East Europe"   ,           
  "region4Spain and Portugal"   ,           "region4Stateless"    ,                   "region4Sub-Saharian Africa"  ,          
 "region4Western Europe" )
reg_ate <- list()
for (j in reg_list) { 
  ate.j = average_treatment_effect(forest.all, subset = 1==X1[,j])
  reg_ate[[j]] <- ate.j |> mutate(region=j)
}
j <-"Americas & Caribbean"
ate.j = average_treatment_effect(forest.all, subset = dta$region4==j)
reg_ate[[j]] <- ate.j |> mutate(region=j)
reg_ate <- bind_rows(reg_ate)

p0 <- reg_ate |>
  bind_rows(tau.hat |> mutate(region="All")) |>
  mutate(region=str_replace(region,"region4",""),
         region=case_when(region=="Central-East Europe"~"CEE",
                        region=="Germany and Austria"~"Germany/\nAustria",
                        region=="Middle Easy and Northern Africa"~"MENA",
                        region=="South-East Europe"~"SEE",
                        region=="Spain and Portugal"~"Spain/\nPortugal",
                        region=="Stateless"~"Stateless",
                        region=="Sub-Saharian Africa"~"SSA",
                        region=="Western Europe"~"WE",
                        region=="Americas & Caribbean"~"Americas",
                        TRUE~region
                        )
         ) |>
  mutate(contrast=case_when(contrast=="1 - 0"~"Complexity",
                            contrast=="2 - 0"~"Requirements",
                            contrast=="3 - 0"~"Welcome",
  )) |>
  mutate(region=fct_relevel(region,"All")) |> 
  ggplot() +
  geom_pointrange(aes(x=region,y=estimate,ymin=estimate-1.96*std.err,
                          ymax=estimate+1.96*std.err,col=contrast,
                      linetype=contrast,
                      shape=contrast),
                  linewidth=1.0,size=.7,
                  position=position_dodge(width = .5)) +
  geom_hline(yintercept=0,alpha=.3)+
  theme_classic() +
  scale_color_discrete("Treatment letter")+
  scale_shape_discrete("Treatment letter")+
  scale_linetype_discrete("Treatment letter")+
  theme(#axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
        text=element_text(size=14))+
  xlab("Nationality region") + ylab("Effect of treatment letter on application rate")
ggsave(paste0(out_folder,"figures/hetero_regions.png"),width=18,height=15,unit="cm")

### heterogeneity by time in ch ################################################
time_in_ch_class <- cut(X1[,"time_in_ch"],breaks=c(10,11,12,16,20,30,67),include.lowest = TRUE)
table(time_in_ch_class)
  
time_in_ch_ate <- list()
for (j in  names(table(time_in_ch_class))) { 
  ate.j = average_treatment_effect(forest.all, subset =  time_in_ch_class == j)
  time_in_ch_ate[[j]] <- ate.j |> mutate(time_in_ch=j)
}
time_in_ch_ate <- bind_rows(time_in_ch_ate)
table(X1[,"time_in_ch"] )

p1 <- time_in_ch_ate |>
  bind_rows(tau.hat |> mutate(time_in_ch="All")) |>
  mutate(contrast=case_when(contrast=="1 - 0"~"Complexity",
                            contrast=="2 - 0"~"Requirements",
                            contrast=="3 - 0"~"Welcome",
                            )) |>
  mutate(time_in_ch=case_when(time_in_ch=="All"~"All",
                              time_in_ch=="[10,11]"~"11",
                              time_in_ch=="(11,12]"~"12",
                              time_in_ch=="(12,16]"~"13-16",
                              time_in_ch=="(16,20]"~"17-20",
                              time_in_ch=="(20,30]"~"21-30",
                              time_in_ch=="(30,67]"~"31\n or more" ),
        time_in_ch=as.factor(time_in_ch),
        time_in_ch=fct_relevel(time_in_ch,"All","11")) |> 
  ggplot() +
  geom_pointrange(aes(x=time_in_ch,y=estimate,ymin=estimate-1.96*std.err,
                      ymax=estimate+1.96*std.err,col=contrast,
                      linetype=contrast,
                      shape=contrast
                      ),
                  linewidth=1.0,size=.7,
                  position=position_dodge(width = .5)) +
  geom_hline(yintercept=0,alpha=.3)+
  theme_classic() +
  scale_color_discrete("Treatment letter")+
  scale_shape_discrete("Treatment letter")+
  scale_linetype_discrete("Treatment letter")+
  theme(#axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
        text=element_text(size=14))+
  xlab("Years in Switzerland") + ylab("Effect of treatment letter on application rate")
ggsave(paste0(out_folder,"figures/hetero_years_in_ch.png"),width=18,height=15,unit="cm")


### heterogeneity by age ################################################
table(X1[,"age"])
age_class <- cut(X1[,"age"],breaks=c(19,25,35,45,55,99),include.lowest = TRUE)
table(age_class)

age_ate <- list()
for (j in  names(table(age_class))) { 
  ate.j = average_treatment_effect(forest.all, subset =  age_class == j)
  age_ate[[j]] <- ate.j |> mutate(age=j)
}
age_ate <- bind_rows(age_ate)
table(X1[,"age"] )

p2 <- age_ate |>
  bind_rows(tau.hat |> mutate(age="All")) |>
  mutate(contrast=case_when(contrast=="1 - 0"~"Complexity",
                            contrast=="2 - 0"~"Requirements",
                            contrast=="3 - 0"~"Welcome",
  )) |>
  mutate(age=case_when(age=="All"~"All",
                        age=="[19,25]"~"19-25",
                              age=="(25,35]"~"26-35",
                              age=="(35,45]"~"36-45",
                              age=="(45,55]"~"46-55",
                              age=="(55,99]"~"56\n or more+" ),
         age=as.factor(age),
         age=fct_relevel(age,"All","19-25")) |> 
  ggplot() +
  geom_pointrange(aes(x=age,y=estimate,ymin=estimate-1.96*std.err,
                      ymax=estimate+1.96*std.err,col=contrast,
                      linetype=contrast,
                      shape=contrast
                      ),
                      linewidth=1.1,size=.8,
  position=position_dodge(width = .5)) +
  geom_hline(yintercept=0,alpha=.3)+
  theme_classic() +
  scale_color_discrete("Treatment letter")+
  scale_shape_discrete("Treatment letter")+
  scale_linetype_discrete("Treatment letter")+
  theme(#axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
        text=element_text(size=14))+
  xlab("Age") + ylab("Effect of treatment letter on application rate")  
ggsave(paste0(out_folder,"figures/hetero_age.png"),width=18,height=15,unit="cm")

library("ggpubr")
ggarrange(p0,p1,legend = 'bottom',common.legend = TRUE)
ggsave(paste0(out_folder,"figures/hetero_both.png"),width=37,height=18,unit="cm",dpi=300)

age_class |>
  as_tibble() |>
  ggplot() +
  geom_bar(aes(y=value)) +
  theme_void()

# merge and assignment
W_best.df <- data.frame(PID=as.numeric(rownames(X2)),W_best=W_best)
dta <- left_join(dta,W_best.df,c("PID"))
dta <- dta %>% 
      mutate(wave2_final=case_when(wave2_arm=="Policy rule"~W_best,
                                   wave2_arm=="Treatment 1"~1,
                                   wave2_arm=="Treatment 2"~2,
                                   wave2_arm=="Treatment 3"~3
                                   ))

# checks
dta_check <- dta %>% 
  filter(wave2) %>%
  dplyr::select(PID,idx,treated,wave2_arm,W_best,wave2_final)
with(dta_check,table(wave2_arm,wave2_final))
with(dta_check,table(wave2_final,W_best,wave2_arm))

## balance check
balance_test <- function(x,data) {
  feols(as.formula(paste(x,"~wave2_arm")),data=data,cluster="idx") %>% summary()
}
bal_list <- list()
bal_list[["time_in_ch"]]<-balance_test("time_in_ch",data=filter(dta,wave2))
bal_list[["time_in_zh"]]<-balance_test("time_in_zh",data=filter(dta,wave2))
bal_list[["age"]]<-balance_test("age",data=filter(dta,wave2))
bal_list[["Americas"]]<-balance_test("I(region4=='Americas & Caribbean')",data=filter(dta,wave2))
bal_list[["Asia"]]<-balance_test("I(region4=='Asia')",data=filter(dta,wave2))
bal_list[["CEE"]]<-balance_test("I(region4=='Central-East Europe')",data=filter(dta,wave2))
bal_list[["GerAus"]]<-balance_test("I(region4=='Germany and Austria')",data=filter(dta,wave2))
bal_list[["Italy"]]<-balance_test("I(region4=='Italy')",data=filter(dta,wave2))
bal_list[["MiddleEast"]]<-balance_test("I(region4=='Middle Easy and Northern Africa')",data=filter(dta,wave2))
bal_list[["SEE"]]<-balance_test("I(region4=='South-East Europe')",data=filter(dta,wave2))
bal_list[["Spain"]]<-balance_test("I(region4=='Spain and Portugal')",data=filter(dta,wave2))
bal_list[["Stateless"]]<-balance_test("I(region4=='Stateless')",data=filter(dta,wave2))
bal_list[["Africa"]]<-balance_test("I(region4=='Sub-Saharian Africa')",data=filter(dta,wave2))
bal_list[["WE"]]<-balance_test("I(region4=='Western Europe')",data=filter(dta,wave2))
bal_list[["idx_count"]]<-balance_test("idx_count",data=filter(dta,wave2) %>% group_by(idx) %>% mutate(idx_count=n()))
etable(bal_list[1:5])
etable(bal_list[6:10])
etable(bal_list[11:15])

# export
dta_wave2 <- dta %>% 
  filter(wave2) %>%
  dplyr::select(PID,idx,wave2_arm,wave2_final) %>%
  rename(Cluster=idx,Treat_arm=wave2_arm,Allocated_treat_num=wave2_final) %>%
  mutate(Allocated_treat=case_when(Allocated_treat_num==0~"None",
                                   Allocated_treat_num==1~"Complexity barrier",
                                   Allocated_treat_num==2~"Knowledge of requirements barrier",
                                   Allocated_treat_num==3~"Welcome letter"
                                   ))
write_csv(dta_wave2,file=paste0(out_folder,"/Treatment_allocation_April2022.csv"))

with(dta_wave2,table(Treat_arm,Allocated_treat))
with(dta_wave2,table(Allocated_treat))


# export split by treatment
dta0 <- read_csv(paste0(data_processed_folder ,"/FullSample_withRandom_applied_movers.csv"))
dta0 <- inner_join(dta0,dta_wave2,c("PID"))
dta0 <- dta0 %>% dplyr::select(PID,last_name,given_name,first_name,street,house_number,
                                PLZ,city,gender,dob,arrive_CH,arrive_ZH,nat,
                               idx,Treat_arm,Allocated_treat_num,Allocated_treat) 
dta0 <- dta0 %>%
        rename(Cluster=idx )
write_excel_csv(filter(dta0,Allocated_treat_num==1),
                file=paste0(out_folder,"/Treatment_allocation_April2022_Treat1.csv"))
write_excel_csv(filter(dta0,Allocated_treat_num==2),
                file=paste0(out_folder,"/Treatment_allocation_April2022_Treat2.csv"))
write_excel_csv(filter(dta0,Allocated_treat_num==3),
                file=paste0(out_folder,"/Treatment_allocation_April2022_Treat3.csv"))
