#####

## Set working directories
data_raw_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data_raw/")
data_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data/")
outputs_path <- ("/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Outputs/")

# Load packages
library(data.table)
library(readxl)
library(lubridate)
library(stringr)
library(dplyr)


# read in the October 11 2021 to 31 March 2022 data and convert to data.table
# Note: the data is badly named by the Canton, the dates are actually 11.10.2021-31.03.2022
apps_1 <- read_xlsx(paste0(data_raw_path,"01-Gesuchseingaenge_09.10.2021_31.01.2022 Zuerich.xlsx"))

# read in the 1 April 2022 until 9 May 2022 data and convert to data.table. We got this data only on 09.08.2022, so added later)
apps_2 <- read_xlsx(paste0(data_raw_path,"Gesuchseingänge_01.04.2022_09.05.2022 Zürich.xlsx"))


# read in the May 10 (including) to December 9 (including) - This constitutes Phase 2 of the treatment period. We received this data on December 13, 2022
apps_3 <- read_xlsx(paste0(data_raw_path,"Gesuchseingänge_09.05.2022_09.12.2022 Zürich.xlsx"))
# delete those from the 9th of May 2022, because we already have them in apps_2
apps_3 <- apps_3 %>% filter(EINGANG!=ymd("2022-05-09"))


common_cols <- intersect(colnames(apps_1), colnames(apps_2))
common_cols <- intersect(common_cols, colnames(apps_3))


apps <- rbind(
  subset(apps_1, select = common_cols),
  subset(apps_2, select = common_cols),
  subset(apps_3, select = common_cols))

setDT(apps)

# create easier to use column names
colnames(apps)
replace_names <- c(
  "FIRMA" = "company",               # takes only one value
  "GGRP" = "ggrp",                   # takes only one value
  "GESCH_JAHR" = "app_year",
  "GESCH_NR" = "app_nr",
  "EINGANG" = "app_date",
  "NAME" = "last_name",
  "VORNAME" = "first_name",
  "GEBURTSDATUM" = "dob",
  "GESCHLECHT" = "gender",
  "ZIVILSTAND" = "marital",
  "PARTEIART" = "party_type" ,         # BEW=applicant, KND=child, EHP=spouse
  "STRASSE" = "street_house_address",
  "ZUSATZ" = "zusatz",
  "PLZ" = "PLZ",
  "ORT" = "city",
  "LAND_CODE" = "nat_code",
  "NATION" = "nat",
  "PARTEIART_GEM" = "party_type_gem",  # takes only one value
  "GEM_NR" = "mun_nr" ,                # takes only one value
  "GEMEINDE" = "municipality"          # takes only one value
)

setnames(apps, old = names(replace_names), new = replace_names)

# change country names from the application data to match the administrative data
corrections <- c(
  "Aegypten"	=	"Ägypten",
  "Aethiopien"	=	"Äthiopien",
  "Bangladesh"	=	"Bangladesch",
  "Bosnien und Herzegowina"	=	"Bosnien-Herz.",
  "China"	=	"China, Volksr.",
  "Dominikanische Republik"	=	"Dominik. Rep.",
  "Moldova"	=	"Moldau",
  "Österreich"	=	"Österreich",
  "Serbien und Montenegro"	=	"Serbien",
  "Tschechische Republik"	=	"Tschechische R.",
  "Vereinigte Staaten"	=	"USA"
)


apps[nat %chin% names(corrections), nat := corrections[nat]]

# reformat the date variables
apps$dob <- as.Date(apps$dob, "CET")      # date was in POSIXct format
#apps[ , dob := dmy(dob)]
for(j in c("app_date")) set(apps, j = j, value = as.Date(apps[[j]]))
# in year-month-day format


# Drop people that were not 18 by mid 2021 (hence, not in the study)
apps_young <- apps[dob > ymd("2003-06-30")]    # keep just because
apps <- apps[dob <= ymd("2003-06-30")]



# correct streets that are in the form "XXstr." to "XXstrasse" / "XXStr." to "XXStrasse"
apps$street_house_address<- str_replace(apps$street_house_address, "str. ", "strasse ")
apps$street_house_address<- str_replace(apps$street_house_address, "Str. ", "Strasse ")

# create some simple versions of key variables
for(j in c("first_name", "last_name", "street_house_address")){
  set(apps, j = paste0(j, "_S"), value = tolower(stringi::stri_trans_general(apps[[j]], "latin-ascii")))}

# change gender to match admin data: from M to männlich and from F to weiblich
apps$gender<- str_replace(apps$gender, "M", "männlich")
apps$gender<- str_replace(apps$gender, "F", "weiblich")




#----------
# Begin matching with admin data

#read in the data from the admin 
admin <- fread(paste0(data_path,
                      "FullSample_withRandom.csv"),
               colClasses = "character"
)



admin$dob <- as.Date(admin$dob, format = "%Y-%m-%d")

# create some simple versions of key variables, to match apps dataframe
for( j in c("first_name_S", "last_name_S")) set(admin, j = j, value = tolower(admin[[j]]))
set(admin, j = "street_house_address", value = tolower(stringi::stri_trans_general(admin$street_house_address, "latin-ascii")))

# Easy try: first and last names (accent stripped & all lower-case versions)
apps[ , matched_1 := paste0(first_name_S, " ", last_name_S) %chin% admin[,paste0(first_name_S, " ", last_name_S)]]
apps[ , .N, matched_1] # gets about 28% of the people (642/2298)

# look for matches based on name distance & dob
test <- adist(
  x = apps[ , paste0(last_name_S, ", ", first_name_S, ", ", dob)],
  y = admin[ , paste0(last_name_S, ", ", first_name_S, ", ", dob)]
)

dtf <- as.data.table(test)
best_matches <- apply(dtf, 1, function(x) which(x == min(x), useNames = F))
match_quality <- apply(dtf, 1, function(x) min(x))
reps <- sapply(best_matches, length)
apps_cols <- unlist(mapply(rep, x = 1:apps[,.N], times = reps, SIMPLIFY = T))
match_quals <- unlist(mapply(rep, x = match_quality, times = reps, SIMPLIFY = T))

# create indicies for matching
apps[ , index := .I]
admin[ , index := .I]

multi_match <-
  cbind(
    apps[apps_cols, .(
      "f_name_app" = first_name_S,
      "l_name_app" = last_name_S,
      "dob_app" = dob,
      "nat_app" = nat,
      "gender_app" = gender,
      "add_app" = street_house_address_S,
      # "arrive_ZH_app" = arrive_ZH,
      # "arrive_CH_app" = arrive_CH,
      "match_names" = matched_1,
      # "match_name_dob" = matched_2,
      "match_quals" = match_quals,
      "index_app" = index#,
      # "letter_1_app" = letter_1,
      # "letter_2_app" = letter_2
    )],
    admin[unlist(best_matches), .(
      "f_name_adm" = first_name_S,
      "l_name_adm" = last_name_S,
      "dob_adm" = dob,
      "nat_adm" = nat,
      "gender_adm" = gender,
      "add_adm" = street_house_address,
      # "arrive_ZH_adm" = arrive_ZH,
      # "arrive_CH_adm" = arrive_CH,
      "index_adm" = PID
    )]
  )


# create distance variable between two datasets based on birthdate
multi_match[ , dob_dist := abs(dob_app - dob_adm)] 


# create dummy variable =1 if nationality is the same
multi_match[!is.na(nat_app) , nat_match := nat_app == nat_adm ] 
# aggregate the number of matches
multi_match[ , N_matches := uniqueN(index_adm), by = .(index_app)]
## create dummy variable =1 if gender is the same
multi_match[ , gender_match := gender_app == gender_adm]

setcolorder(multi_match, neworder = c("index_app", "f_name_app", "f_name_adm", "l_name_app", "l_name_adm", "dob_app", "dob_adm", "dob_dist", "match_quals", "nat_app",  "nat_adm", "gender_app", "gender_adm", "add_app", "add_adm"))
setkey(multi_match, N_matches, match_quals, index_app)


# cases:
# If name and birthdate match perfectly, accept. 
# If only slight name/birthdate differences but matching addresses, accept.
# I checked this by looking manually - among those not yet confirmed (after Step 1 below), sort by lowest dob_dist, and check manually at names (sometimes middle names were different or not reported in one of the two datasets) and then check addresses
# potential exclusion: if there are some close matches with small address differences

set(multi_match, j = "confirmed_match", value = FALSE)
multi_match[match_quals %in% c(0,1,2, 4, 5), confirmed_match := T] # Step 1
multi_match[index_app %in% c(450, 704, 1107, 479, 1031, 1065, 1067,567,623,896,1043,643,1194, 1235, 1996, 1611, 1881, 2601, 2694, 2730, 1939, 2620, 2865, 1874, 1657, 2394), confirmed_match := T]
# same name, birthday, address, women, and different last name -- perhaps they changed their last name by getting married
multi_match[index_app %in% c(1263, 426), confirmed_match := T]
# Step 2
multi_match[dob_dist %in% c(6935,1675,912), confirmed_match := F] # These don't match



# Set up the merge of correct matches
dim(apps)
apps_merge <- merge(apps, multi_match[confirmed_match == T, .(index_app, index_adm)], all.x = T, all.y = F, by.x = "index", by.y = "index_app")
setnames(apps_merge, old = "index", new = "index_app")
dim(apps_merge)

# remove some of the helper columns before saving the processed file for merging
for(j in c("first_name_S", "last_name_S", "street_house_address_S", "matched_1", "index_app")) set(apps_merge, j = j, value = NULL)

# create variable "applied" indicating whether the individual applied (and is in our original sample) or not
apps_merge[!is.na(index_adm), applied:=1]
apps_merge[is.na(index_adm), applied:=0]

# Save a file of those who applied between October 2021 and December 2022, with an indicator=1 if they were part of our sample
fwrite(apps_merge, paste0(data_path, "app_oct_2021_dec_2022.csv"))



#---------- Prepare admin data

# Merge those who applied back onto the original admin data
admin_merge <- merge(admin, apps_merge, all.x = T, all.y = F, by.x = "PID", by.y = "index_adm")
admin_merge[ , dob_dist := abs(lubridate::ymd(dob.x) - lubridate::ymd(dob.y))]
#
# # remove some of the helper columns before saving the processed file for merging
admin_merge = subset(admin_merge, select = -c(last_name.y, first_name.y, dob.y, gender.y, company, ggrp, gender.y, marital, party_type, street_house_address.y, zusatz, PLZ.y, city.y, nat_code, nat.y, party_type_gem, mun_nr) )
#

## Housekeeping and variable creation
# transform applied into 0 or 1 (now only NA or 1)
admin_merge$applied <- ifelse(is.na(admin_merge$applied), 0, admin_merge$applied )

# Create variable =1 if part of Wave 1 (October 11 2021-May 9 2022 -- including both) or =2 if part of Wave 2 (May 10 2022-December 9 2022 -- including both)
# created from the old variable "treated" (treated=1) which referred to the first treatment period (i.e. Wave 1)
admin_merge$wave <- ifelse(admin_merge$treated==1, 1 , 2)

# Create factor for Treatment in Wave 1 (“complexity”, “welcome”, “requirements”, “none”)
# "none" corresponds to control group, i.e. treated in Wave 2
admin_merge$treatmentWave1 <- case_when(admin_merge$treatment_1==1 ~ "complexity",
                                        admin_merge$treatment_2==1 ~ "requirements",
                                        admin_merge$treatment_3==1 ~ "welcome",
                                        TRUE ~ "none")

# Read in and merge Treatment allocation data for Wave 2, determined by Achim for the policy rule/random allocation
allocation_wave2 <- read.csv(paste0(outputs_path,'Treatment_allocation_April2022.csv')) %>% select(PID, Allocated_treat_num, Treat_arm) 
allocation_wave2$PID <- as.character(allocation_wave2$PID)

admin_merge <- left_join(admin_merge, allocation_wave2, by='PID')

# Create factor for Treatment in Wave 2 (“complexity”, “welcome”, “requirements”, “none”)
# "none" corresponds to having been treated in Wave 1
admin_merge$treatmentWave2 <- case_when(admin_merge$Allocated_treat_num==1 & admin_merge$wave==2 ~ "complexity",
                                        admin_merge$Allocated_treat_num==2 & admin_merge$wave==2 ~ "requirements",
                                        admin_merge$Allocated_treat_num==3 & admin_merge$wave==2 ~ "welcome",
                                        TRUE ~ "none")


# Policy Rule or Random Assignment in Wave 2
# gets values (“random”, “policy tree”, NA) --- NA corresponding to Wave 1
admin_merge$policyRule <- case_when(admin_merge$Treat_arm=="Treatment 1" & admin_merge$wave==2 ~ "random",
                                    admin_merge$Treat_arm=="Treatment 2" & admin_merge$wave==2 ~ "random",
                                    admin_merge$Treat_arm=="Treatment 3" & admin_merge$wave==2 ~ "random",
                                    admin_merge$Treat_arm=="Policy rule" & admin_merge$wave==2 ~ "policy tree",
                                    TRUE ~ "NA")


# Region 4
region_list <- read.csv(paste0(data_raw_path, "region_list.csv")) %>% select(region4, iso_code)
admin_merge <- left_join(admin_merge, region_list, by="iso_code")


# Drop variable treated and treatments
admin_merge <- admin_merge %>% select(-treated,
                                      -treatment_1,
                                      -treatment_2,
                                      -treatment_3,
                                      -first_name_S,
                                      -last_name_S, 
                                      -group_1, 
                                      -control, 
                                      -municipality, 
                                      -dob_dist,
                                      -Allocated_treat_num,
                                      -Treat_arm)




# # # add column for those who moved
movers <- fread(paste0(data_path,"FullSample_withRandom_applied_movers.csv"))

movers <- movers %>% 
  select(PID,movers)

#
admin_merge <- left_join(admin_merge |> mutate(PID=as.integer(PID)), movers, by= 'PID')

# Note that there are some people who are not treated in either waves: these are 101 individuals who moved out of the municipality during Wave 1 (movers==1) or 99 who were in the control group in Wave 1 but applied

# Clean before saving
admin_merge <- admin_merge %>% 
  rename('dob'='dob.x',
         'first_name' = 'first_name.x',
         'last_name' = 'last_name.x',
         'city' = 'city.x',
         'gender' = 'gender.x',
         'nat' = 'nat.x',
         'PLZ' = 'PLZ.x',
         'street_house_address' = 'street_house_address.x')

## Save file
fwrite(admin_merge, paste0(data_path,"FullSample_withRandom_applied_full_wave2.csv"))


