
rm(list=ls())

library("tidyverse")
library("readr")
library("policytree")
library("grf")
library("glmnet")
library("glmnetUtils")
library(DiagrammeR)
library("DiagrammeRsvg")
library("foreach")
library("parallel")
library("readxl")
library("randomizr")
library("fixest")
library("estimatr")
library("tidylog")
library("xtable")

setwd("/Users/kahrens/MyProjects/policy_tree")

data_folder <- "/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data_raw"
out_folder <- "/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Outputs"
data_processed_folder <- "/Volumes/pp/projects/zh-citizenship-21/Policy_tree/Data"
paper_out_path <- "/Users/kahrens/MyProjects/policy_tree_paper/"

dta <- read_csv(paste0(data_processed_folder,"/FullSample_withRandom_applied_full_wave2.csv"))

source("./Code/auxiliary_functions.R")


# there are two cases with two applications. just consider the first. 
dta <- dta |> 
    group_by(PID) |>
    arrange(app_date,.by_group=TRUE) |>
    summarise_all(first) |>
    ungroup()

with(dta,table(wave,treatmentWave1))
with(dta,table(wave,treatmentWave2))

# create a panel with person-wave observations
# all individuals are in the first wave
# second wave only includes those that have not applied by XXXX

## wave 1
dta_wave1 <- dta |> 
  mutate(wave_period=1) |> 
  filter(!movers)   # only non-movers

## wave 2
dta_wave2_movers <- read_excel(paste0(data_folder,"/control_group_bereinigt_2023.xlsx"))
dta_wave2 <- dta |> filter(wave==2) |> 
  mutate(wave_period=2) |> 
  filter(!movers) |> # did not move in 1st wave
  filter(app_date>=as.Date("2022-05-10") | is.na(app_date)) # only individuals that haven't applied by start of wave 2
dta_wave2 <- dta_wave2 |>
  filter(PID %in% dta_wave2_movers$PID)

# combine
dta <- bind_rows(dta_wave1,dta_wave2 
                )
dta <- dta |> select(-applied,-wave)

sum(dta$wave_period==1)
sum(dta$wave_period==2)

# create application outcome using application date
dta <- dta |>
    mutate(applied=case_when(wave_period==1&!is.na(app_date)~app_date<as.Date("2022-05-10"),
                            wave_period==2&!is.na(app_date)~app_date<as.Date("2022-12-10"),
                            is.na(app_date)~FALSE
                            ))  
dta <- dta |>
    mutate(applied=applied*100)

# create treatment variable
dta <- dta |>
    mutate(treatment=if_else(wave_period==1,treatmentWave1,treatmentWave2))
dta <- dta |> select(-treatmentWave1,-treatmentWave2)
with(dta,table(wave_period,treatment))

# policy variables
with(dta,table(wave_period,policyRule))
dta <- dta |> 
    mutate(policy_rule=case_when(wave_period==1&treatment=="none"~"Nothing",
                                 wave_period==1&treatment %in% c("complexity","requirements","welcome")~"Random",
                                 wave_period==2&policyRule=="policy tree"~"Policy tree",
                                 wave_period==2&policyRule=="random"~"Random"
                                 ),
           policy_rule=as.factor(policy_rule),
           policy_rule=fct_relevel(policy_rule,"Nothing")
           )
with(dta,table(wave_period,policy_rule))
dta <- dta |> 
  mutate(policy_rule2=case_when(wave_period==1&treatment=="none"~"Nothing",
                               policyRule=="policy tree"~"Policy tree",
                               treatment=="complexity"~"Always Complexity",
                               treatment=="requirements"~"Always Requirements",
                               treatment=="welcome"~"Always Welcome"
                               ),
         policy_rule2=as.factor(policy_rule2),
         policy_rule2=fct_relevel(policy_rule2,"Nothing"),
         treatment = as.factor(treatment),
         treatment=fct_relevel(treatment,"none")
         )
with(dta,table(wave_period,policy_rule2))
dta <- dta |> dplyr::select(-policyRule)

dta <- dta |>
  rename(age=age_y,
         time_in_zh=time_zh_y,
         time_in_ch=time_ch_y) |>
  mutate(time2=wave_period==2)


dta <- dta |> 
  mutate(region5=region4,
         region5=if_else(nat %in% c("USA","Kanada"),"US/Canada",region5),
         region5=if_else(region5=="Americas & Caribbean","Other Americas & Caribbean",region5)
         )

with(filter(dta,wave_period==2),table(policy_rule2,applied))


write_csv(dta,file=paste0(data_processed_folder,"/Final_for_analysis.csv"))

###############################################################
### descriptives                                            ###
###############################################################

library("fastDummies")

filter(dta,wave_period==1) |> dim()

filter(dta,wave_period==1) |>
  dummy_cols(select_columns = "region4") |>
  select(time_in_ch,time_in_zh,age,female,applied,starts_with("region4_")) |>
  tidyr::pivot_longer(-0) |> 
  mutate(name=case_when(name=="time_in_ch"~"Years in Switzerland",
                        name=="time_in_zh"~"Years in Zurich",
                        name=="age"~"Age",
                        name=="female"~"Female",
                        name=="applied"~"Naturalization application",
                        TRUE~name
                        ),
         name=str_replace(name,"region4\\_",""),
         name=str_replace(name,"\\&","&")
         ) |>
  group_by(name) |>
  summarise(avg= mean(value),
            sd = sd(value),
            min=min(value),
            max=max(value)
            ) |>
  mutate(sort=case_when(name=="Age"~1,
                        name=="Years in Switzerland"~1,
                        name=="Years in Zurich"~1,
                        name=="Female"~1,
                        name=="Naturalization application"~1,
                        TRUE~9
                        )) |>
  arrange(sort,name)
  xtable() |>
  print(include.rownames=F)

###############################################################
### estimation                                              ###
###############################################################

table(dta$policy_rule)

summary(fit1 <- feols(applied~policy_rule2+region4+age+time_in_ch+time_in_zh+female,data=filter(dta,wave_period==2),cluster=~idx))
summary(fit2 <- feols(applied~policy_rule +region4+age+time_in_ch+time_in_zh+female,data=filter(dta,wave_period==2),cluster=~idx))

summary(fit3 <- feols(applied~policy_rule2+wave_period+region4+age+time_in_ch+time_in_zh+female,data=dta,cluster=~idx))
summary(fit4 <- feols(applied~policy_rule+wave_period+region4+age+time_in_ch+time_in_zh+female,data=dta,cluster=~idx))

###############################################################
### export                                                  ###
###############################################################

mydict <- c(`policy_rule2Always Complexity`="Always Complexity",
            `policy_rule2Always Requirements` ="Always Requirements"  ,   
            `policy_rule2Always Welcome` ="Always Welcome"  ,
            `policy_rule2Policy tree`  ="Policy Tree"  , 
            `policy_rulePolicy tree`   ="Policy Tree"   ,
            `policy_ruleRandom`       ="Random" ,  
            policy_rule2Nothing  ="Nothing",
            time2         ="Wave 2",
            policy_ruleNothing ="Nothing"           
)

est_list <- list(fit1,fit2,fit3,fit4)

etable(est_list,
       dict=mydict,
       float=FALSE,
       keep=c("Alw","Pol","Nothing","Random","Wave"),
       extralines=list(
         "Policy tree - Always 1"=unlist(lapply(est_list,function(x) difftest("Policy tree","Complexity",x)$str)),
         "Policy tree - Always 2"=unlist(lapply(est_list,function(x) difftest("Policy tree","Requirements",x)$str)),
         "Policy tree - Always 3"=unlist(lapply(est_list,function(x) difftest("Policy tree","Welcome",x)$str)),
         "Policy tree - Random"= unlist(lapply(est_list,function(x) difftest("Policy tree","Random",x)$str)),
         "Policy tree - Nothing"=unlist(lapply(est_list,function(x) difftest("Policy tree","Nothing",x)$str)),
         "Waves"=c(rep("Wave 1",2),rep("Waves 1-2",4)),
         "Estimator"=c(rep("OLS",2),rep("OLS",2),rep("PDS-Lasso",2)) #,rep("DDML",2))
       )#,
       #tex=TRUE
)
