# Achim Ahrens 
# auxiliary programs for "policytree" analysis

# The conditional response surfaces mu_k(X) for a single outcome can be reconstructed from
# the contrasts tau_k(x), the treatment propensities e_k(x), and the conditional mean m(x).
# Given treatment "A" as baseline we have:
# m(x) := E[Y | X] = E[Y(A) | X] + E[W_B (Y(B) - Y(A))] + E[W_C (Y(C) - Y(A))]
# which given unconfoundedness is equal to:
# m(x) = mu(A, x) + e_B(x) tau_B(X) + e_C(x) tau_C(x)
# Rearranging and plugging in the above expressions, we obtain the following estimates
# * mu(A, x) = m(x) - e_B(x) tau_B(x) - e_C(x) tau_C(x)
# * mu(B, x) = m(x) + (1 - e_B(x)) tau_B(x) - e_C(x) tau_C(x)
# * mu(C, x) = m(x) - e_B(x) tau_B(x) + (1 - e_C(x)) tau_C(x)




get_oos_reward <- function(fit,x1,y1,x2,mode=4) {
  
  tau.hat <- predict(fit,newdata=x2)$predictions[,,]
  if (mode==3) {
    W.hat <- c(0.3,0.3,0.3)
    names(W.hat) <- as.character(1:3)
  } else {
    W.hat <- c(2/5,1/5,1/5,1/5)
    names(W.hat) <- as.character(0:3)
  }
  
  # get E[Y|X]
  Y.hat.fit <- regression_forest(x1,y1)
  Y2.hat <- predict(Y.hat.fit,newdata=x2)$predictions
  
  ## mu
  if (mode==3) {
    muA <- Y2.hat - W.hat["2"] * tau.hat[, "2 - 1"] - W.hat["3"] * tau.hat[, "3 - 1"]
    muB <- Y2.hat + (1 - W.hat["2"]) * tau.hat[, "2 - 1"] - W.hat["3"] * tau.hat[, "3 - 1"]
    muC <- Y2.hat - W.hat["2"] * tau.hat[, "2 - 1"] + (1 - W.hat["3"]) * tau.hat[, "3 - 1"]
    mu.hat <- cbind(muA,muB,muC)
    colnames(mu.hat) <- as.character(1:3)    
  } else {
    muA <- Y2.hat - W.hat["1"] * tau.hat[, "1 - 0"]       - W.hat["2"] * tau.hat[, "2 - 0"]       - W.hat["3"] * tau.hat[, "3 - 0"]
    muB <- Y2.hat + (1 - W.hat["1"]) * tau.hat[, "1 - 0"] - W.hat["2"] * tau.hat[, "2 - 0"]       - W.hat["3"] * tau.hat[, "3 - 0"]
    muC <- Y2.hat - W.hat["1"] * tau.hat[, "1 - 0"]       + (1 - W.hat["2"]) * tau.hat[, "2 - 0"] - W.hat["3"] * tau.hat[, "3 - 0"]
    muD <- Y2.hat - W.hat["1"] * tau.hat[, "1 - 0"]       - W.hat["2"] * tau.hat[, "2 - 0"]       + (1 - W.hat["3"]) * tau.hat[, "3 - 0"]
    mu.hat <- cbind(muA,muB,muC,muD)
    colnames(mu.hat) <- as.character(0:3)   
  }

  return(list(mu.hat=mu.hat,y.hat=Y2.hat))
  
}

policy_multiply <- function(G,W) {
  Y <- case_when(W=="1"~G[,1],W=="2"~G[,2],W=="3"~G[,3])
  return(Y)
}
policy_multiply4 <- function(G,W) {
  Y <- case_when(W=="0"~G[,1],W=="1"~G[,2],W=="2"~G[,3],W=="3"~G[,4])
  return(Y)
}

myrandom <- function(n) as.factor(x=sample(c("1", "2", "3"), size=n, prob=c(1/3,1/3,1/3), replace = TRUE))


myround <- function(x,digits) format(round(x,digits),nsmall=digits)

difftest <- function(x1, x2, fit){
  vars <- names(coef(fit))
  if (is.character(x1)) x1 <- which(str_detect(vars,x1))
  if (is.character(x2)) x2 <- which(str_detect(vars,x2))
  if (length(x1)==0 | length(x2)==0) {
    return(list(str=""))
  } else {
    V <- vcov(fit)
    b <- as.numeric(coef(fit))
    diffest <- b[x1]-b[x2]
    vardiff <- (V[x1,x1] +
                  V[x2,x2]) - (2*(V[x1, x2]))
    # variance of x1 + variance of x2 - 2*covariance of x1 and x2
    diffse <- sqrt(vardiff)
    tdiff <- (diffest)/(diffse)
    pval <- 2*(1-pnorm(abs(tdiff)))
    upr <- diffest + 1.96*diffse # will usually be very close to 1.96
    lwr <- diffest + 1.96*diffse
    return(list(est=diffest,
                t=tdiff,
                se=diffse,
                p=pval,
                lwr=lwr,
                upr=upr,
                str=paste0(myround(diffest,4),case_when(pval<0.01~"***",pval<0.05~"**",pval<0.1~"*",TRUE~"")," (",myround(diffse,4),")")
                )
    )
  }
}
# 
# difftable <- function(fit,xtable=TRUE,newnames=NULL,...) {
#   vars <- names(coef(fit))
#   p <- length(vars)
#   out <- matrix(rep(NA,p^2),ncol=p)
#   for (i in 1:p) {
#     for (j in 1:p) {
#       if (i==j) {
#         out[i,j] <- "--"
#       } else {
#         out[i,j]<-difftest(i,j,fit)$str
#       }
#     }
#   }
#   if (xtable) {
#     require(xtable)
#     rownames(out) <- colnames(out) <- newnames
#     out <- xtable(out)
#     print.xtable(out,
#                  only.contents=TRUE,
#                  floating=FALSE,
#                  include.rownames=!is.null(newnames),
#                  include.colnames=!is.null(newnames),
#                  hline.after=NULL,
#                  ...)
#   }
#   return(out)
# }


ipw.multi_arm_causal_forest <- function(object, outcome = 1, ...) {
  outcome <- outcome[1]
  if (!outcome %in% 1:NCOL(object$Y.orig)) {
    outcome <- match(outcome, colnames(object$Y.orig, do.NULL = FALSE))
    if (is.na(outcome)) {
      stop("`outcome` should be a column number/name specifying a column in Y.")
    }
  } else {
    outcome <- as.integer(outcome)
  }
  treatment.names <- levels(object$W.orig)
  num.treatments <- length(treatment.names)
  num.samples <- nrow(object$W.hat)
  observed.treatment <- match(object$W.orig, treatment.names)
  observed.treatment.idx <- cbind(seq_along(object$W.orig), observed.treatment)
  
  mu.matrix <- conditional_means(object, outcome)
  YY <- matrix(0, num.samples, num.treatments)
  IPW <- matrix(0, num.samples, num.treatments)
  YY[observed.treatment.idx] <- object$Y.orig[, outcome]
  IPW[observed.treatment.idx] <- 1 / object$W.hat[observed.treatment.idx]
  Gamma.matrix <- (YY  ) * IPW  
  
  Gamma.matrix
}

